/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <math.h>
#include "nrutil.h"
#include "util.h"
#include "random.h"
#include "patlookup.h"
#include "parameters.h"
#include "data.h"
#include "prior.h"
#include "myio.h"
#include "chain.h"
#include "patutil.h"
/*******************************************************************/
/* A function to create a copy of the structure c and to return it */
/*******************************************************************/
chain copy_chain(chain *c)
{
	chain tmp;

	tmp.loci=c->loci;
	copyprior(&tmp.muprior,c->muprior);
    tmp.fathers=copy_parents(&c->fathers);
	tmp.mothers=copy_parents(&c->mothers);
	tmp.sampleallparentaloffspring=c->sampleallparentaloffspring;
//	tmp.proploc=copy_dvector(c->proploc,0,c->loci);
	tmp.mu=c->mu;
	tmp.muindex=c->muindex;tmp.temperindex=c->temperindex;
	tmp.temper=c->temper;
	tmp.temper3=c->temper3;

	return tmp;
}
/*******************************************************************/
void destroy_chain(chain *c)
{
	destroy_parents(&c->fathers);
	destroy_parents(&c->mothers);
//	free_dvector(c->proploc,0);
	c->temper=c->temper3=0.0;
	c->temperindex=0;
	c->mu=0.0;
	c->muindex=0;
}
/*****************************************************/
void write_chain(FILE *out, chain *c)
{
	fprintf(out,"mutation.rate: %g\n",c->mu);
	fprintf(out,"temper: %g\n",c->temper);
	fprintf(out,"index: %d\n",c->temperindex);
	fprintf(out,"muprior: ");
	printprior(out,c->muprior,"\n");
	write_parents(out,"fathers",&c->fathers);
	write_parents(out,"mothers",&c->mothers);
	return;
}
/******************************************************/
chain read_chain(FILE *in)
{
	chain c;
	extern prior nllprior;
	c.mu=double_scan(in,"mutation.rate",0.0001);
	c.temper=double_scan(in,"temper",1.0);
	c.temperindex=int_scan(in,"index",1);
	c.muprior=prior_scan(in,"muprior",nllprior);
	c.fathers=read_parents(in,"fathers");
	c.mothers=read_parents(in,"mothers");

	return c;
}
/******************************************************/
chain starting_chain(parameters *p, data *d)
{
	chain c;
//	int i;
	
	if (p->initfile!=NULL) {
		c=read_chain(openinputfile(p->initfile));
	} else {
		printf("Fathers: ");
		c.fathers=assign_parents(p->fatherprior,p->alphaprior,d->fixed_males,
			d->nfixedfathers,p->fatherrange,d->Y);
		if (p->knownfathers) c.fathers.known=copy_ivector(d->knownfathers,0,d->samples);
		else c.fathers.known=NULL;
		printf("Mothers: ");
		c.mothers=assign_parents(p->motherprior,p->betaprior,d->fixed_females,
			d->nfixedmothers,p->motherrange,d->Y);
		if (p->knownmothers) c.mothers.known=copy_ivector(d->knownmothers,0,d->samples);
		else c.mothers.known=NULL;
		copyprior(&c.muprior,p->muprior);
		c.loci=d->loci;
		c.temperindex=1;

		/* sample mu if it not constant (and get a sensible starting value)*/
		if (isconst(&(c.muprior))) c.mu=c.muprior.par[0];
		else {
			sample_prior(&c.mu,c.muprior);
			if (c.mu<1E-6||c.mu>1E-3) c.mu=1E-4;
		}
		c.muindex=1;
		c.temper=1.0;
		c.temper3=1.0;
//		c.proploc=nrdvector(0,c.loci);
//		for (i=1;i<=c.loci;i++) c.proploc[i]=1.0;
//		c.proploc[0]=-10.;
		assign_offspring_random(&c.fathers,&c.mothers);
		if (c.fathers.off.model==3||c.mothers.off.model==3)
			c.sampleallparentaloffspring=sampleallparentaloffspringb;
		else 
			c.sampleallparentaloffspring=sampleallparentaloffspringa;
	}
	return c;
}
/******************************************************/
chain copy_resample_chain(chain *c)
{
	chain tmp;
	tmp=copy_chain(c);
	resample_parents(&tmp.fathers,&tmp.mothers);//,tmp.proploc);
	resample_parents(&tmp.mothers,&tmp.fathers);//,tmp.proploc);
	return tmp;
}
/******************************************************/
int letcode(int a)
{
	if (a>50) return 90;
	else if (a<1) return 45;
	else return a+64;
}
/******************************************************/
void write_sorteddata(FILE *out ,data *d, chain *c,int lettercode)
{
	int i,j,k,itmp;

	for (i=1;i<=c->fathers.n;i++) {
		for (j=1;j<=d->samples;j++) {
			if (c->fathers.parentage[j]==i) {
				for (k=1;k<=d->loci;k++) {
					if (d->Y[j][2*k-1]!=c->mothers.genotype[c->mothers.parentage[j]][2*k-1]||
						d->Y[j][2*k-1]!=c->mothers.genotype[c->mothers.parentage[j]][2*k]) {
						itmp=d->Y[j][2*k];d->Y[j][2*k]=d->Y[j][2*k-1];d->Y[j][2*k-1]=itmp;
					}
					if (lettercode) {
						fprintf(out,"%c-%c ",letcode(d->Y[j][2*k-1]),
							letcode(d->Y[j][2*k]));
					} else {
						fprintf(out,"%d-%d  ",d->allele_labels[d->Y[j][2*k-1]],
							d->allele_labels[d->Y[j][2*k]]);
					}
				}
				fprintf(out,"\n");
			}
		}
		fprintf(out,"\n");
	}
}
/*******************************************************************/
/*  A new version that should work !!!!                            */
/*******************************************************************/
void swapmalesfemales(chain *c)
{
	int i,j,itmp,**im,*iv;

	im=copy_imatrix(c->fathers.genotype,1,c->fathers.n,1,2*c->loci);
	for (i=1;i<=c->mothers.n;i++) {
		for (j=1;j<=2*c->loci;j++) {
			c->fathers.genotype[i][j]=c->mothers.genotype[i][j];
		}
	}
	for (i=1;i<=c->fathers.n;i++) {
		for (j=1;j<=2*c->loci;j++) {
			c->mothers.genotype[i][j]=im[i][j];
		}
	}
	free_imatrix(im,1,1);
	itmp=c->fathers.n;
	c->fathers.n=c->mothers.n;
	c->mothers.n=itmp;

	iv=c->fathers.parentage;
	c->fathers.parentage=c->mothers.parentage;
	c->mothers.parentage=iv;

	i=c->fathers.n>c->mothers.n?c->fathers.n:c->mothers.n;
	for (j=1;j<=i;j++) {
		itmp=c->fathers.offspring[j];
		c->fathers.offspring[j]=c->mothers.offspring[j];
		c->mothers.offspring[j]=itmp;
	}
	
	if (c->fathers.fixed||c->mothers.fixed) 
		myerror("not here in swapmalesfemales");
	if (c->fathers.known||c->mothers.known) 
		myerror("not here in swapmalesfemales");
}
