#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "prior.h"
#include "myio.h"
#include "naiveutil.h"
#include "nrutil.h"
#include "random.h"
#include "patlookup.h"
#include "util.h"

double lpcrypt(int **data, int *mother, double **rho, 
			   int cr, int loci,int *alleles, int *use, int n, int model, long *nfathers);
double allfathers(int *alleles, int loci);

long replicates;

int main(int argc, char *argv[])
{
	int *father,*use;
	long i,success=0,nf=0;
	crypt_model m;
	FILE *out;

	if (argc != 4) {
		printf("use \ncryptic infile outfile seed\n\n");
		exit(0);
	}
	
	m=read_crypt_model(argv[1]);
	out=openoutputfile(argv[2]);
	starTup(atoi(argv[3]));
	
	use=nrivector(1,m.n);
	for (i=1;i<=m.n;i++) use[i]=1;
	printf("\n\nprobability = %g\n",
	exp(lpcrypt(m.data,m.mother,m.rho,1,m.loci,m.alleles,use,m.n,m.freqmodel,&nf)));
	printf("%d possible fathers out of %g\n",nf,allfathers(m.alleles,m.loci));
	free_ivector(use,1);
	return 1;
	
	for (i=1;i<=replicates;i++) {
		father = generate_genotype(m.loci,m.alleles,m.rho,m.freqmodel);
		if (possible_father(father,m.mother,&m)) {
			success+=1;
			write_ivector(out," ",father,1,2*m.loci);
		}
		FREE(father);
	}
	printf("\nprobability of cryptic father = %g\n",(double)success/(double)replicates);
	fclose(out);
	return 1;
}
/************************************************************************************/
/*  Function for calculating the probability of a randomly selected individual      */
/*  being the father, conditional on the given mother                               */
/*  
    data		-  the offspring data
	mother		-  the maternal genotype
	rho			- background frequences (model = 1, or mn dirichlet parameters)		
	cr			-  current locus 
	loci		- number of loci	
	alleles		- number of alleles	
	use			- which individuals in data to consider
	n			- sample size	
	model		- model for background freq, either mn dirichlet (model=0) or mn	
	nf			- count for possible fathers					                   */
/***********************************************************************************/
double lpcrypt(int **data, int *mother, double **rho, 
			   int cr, int loci,int *alleles, int *use, int n, int model, long *nf)
{
	int al[3],*use2,poss,i;
	double p=0.0,lp;
	static long cr_count=0;
	
	if (cr != loci) {
		use2=nrivector(1,n);
		for (al[1]=1;al[1]<=alleles[cr];al[1]++) {
			for (al[2]=al[1];al[2]<=alleles[cr];al[2]++) {
				poss=0;
				for (i=1;i<=n;i++) {
					use2[i]=0;
					if (use[i]) {
						if (!lookup_muts(data[i]+2*cr-2,mother+2*cr-2,al)>0) {
							use2[i]=1;
							poss=1;
						} 
					} 
				}
				if (poss) {  // are there any possibles ?
					if (model==1) {
						if (al[1]==al[2]) lp = 2*log(rho[cr][al[1]]);
						else lp = log(2.)+log(rho[cr][al[1]]) +log(rho[cr][al[2]]);
					} else lp = log_dmulti_dirichletc(al,rho[cr],alleles[cr],2);
					p += exp(lp+lpcrypt(data,mother,rho,cr+1,loci,alleles,use2,n,model,nf));
				}
			}
		}
		free_ivector(use2,1);
	} else {
		for (al[1]=1;al[1]<=alleles[cr];al[1]++) {
			for (al[2]=al[1];al[2]<=alleles[cr];al[2]++) {
				poss=0;
				for (i=1;i<=n;i++) {
					if (use[i]) 
						if (!lookup_muts(data[i]+2*cr-2,mother+2*cr-2,al)>0) 
							poss=1;				
				}
				if (poss) {
					*nf +=1;
					if (model==1) {
						if (al[1]==al[2]) {
							p += exp(2*log(rho[cr][al[1]]));
						} else {
							p += exp(log(2.)+log(rho[cr][al[1]]) +log(rho[cr][al[2]])); 
						}
					} else 	p += exp(log_dmulti_dirichletc(al,rho[cr],alleles[cr],2));
				}
			}
		}
	}
	return log(p);
}
/***********************************************************************************/
double allfathers(int *alleles, int loci)
{
	double tot=1.;
	int i;

	for (i=1;i<=loci;i++) {
		tot *= (double)alleles[i]*(double)(alleles[i]+1.)/2.;
	}
	return tot;
}
/***********************************************************************************/