/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include "util.h"
#include "nrutil.h"
#include "data.h"
#include "parameters.h"
#include "myio.h"

int **Y,loci,*alleles,max_alleles,**allele_labels,freqmodel,samples;
double **rho;

/**********************************************************************/
int destroy_data(data *any)
{
	int i;
	if (any->fixed_males) free_imatrix(any->fixed_males,1,1);
	if (any->fixed_females) free_imatrix(any->fixed_females,1,1);
	for (i=1;i<=any->loci;i++) free_ivector(any->allele_labels[i],1);
	FREE(any->allele_labels);
	free_dmatrix(any->rho,1,1);                    
	free_ivector(any->alleles,1);
	if (any->background_freq) free_imatrix(any->background_freq,1,1);
	free_imatrix(any->Y,1,1);
	return 1;
}
/**********************************************************************/
int **subsetdata(int **Y, int n, int *usen,int loc, int *useloc,int freeY)
{
	int **m,i,j;
	
	/* First check that the subsetting is possible   */
	if (usen) {
		if (usen[0]>n||imax(usen,usen[0])>n)
			myerror("error, usen too long in subsetdata");
	}
	if (useloc) {
		if (useloc[0]>loc||imax(useloc,useloc[0])>loc) {
           myerror("error, useloc too large in subsetdata");
		}
	}
	if (useloc&&usen) {
		m=nrimatrix(1,usen[0],1,2*useloc[0]);
		for (i=1;i<=usen[0];i++) {
			for (j=1;j<=useloc[0];j++) {
				m[i][2*j-1]=Y[usen[i]][2*useloc[j]-1];
				m[i][2*j]=Y[usen[i]][2*useloc[j]];
			}
		}
	} else if (useloc) {
		m=nrimatrix(1,n,1,2*useloc[0]);
		for (i=1;i<=n;i++) {
			for (j=1;j<=useloc[0];j++) {
				m[i][2*j-1]=Y[i][2*useloc[j]-1];
				m[i][2*j]=Y[i][2*useloc[j]];
			}
		}
	} else {
		m=nrimatrix(1,usen[0],1,2*loc);
		for (i=1;i<=usen[0];i++) {
			for (j=1;j<=2*loc;j++) {
				m[i][2*j-1]=Y[usen[i]][j];
				m[i][2*j]=Y[usen[i]][j];
			}
		}
	}
	if (freeY) free_imatrix(Y,1,1);
	return m;
}
/*******************************************************************/
void recode(int **m, int n, int spare, int loci, int *alleles, int **allele_labels)
{
	int locus,i,j;

	if (n==0) return;

	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=n;i++) {
			if (m[i][2*locus-1]>0) {
				for (j=1;j<=alleles[locus]-spare;j++) {
					if (m[i][2*locus-1]==allele_labels[locus][j]) {
						m[i][2*locus-1]=j;
						break;
					}
				}
				if (j==alleles[locus]+1-spare) {
					if (!spare) {
						printf("alleles %d j = %d  val = %d\n",alleles[locus],j,m[i][2*locus-1]);
						myerror("error in recode");
					} else
						m[i][2*locus-1] =j; 
				}
			}
			if (m[i][2*locus]>0) {
				for (j=1;j<=alleles[locus]-spare;j++) {
					if (m[i][2*locus]==allele_labels[locus][j]) {
						m[i][2*locus]=j;
						break;
					}
				}
				if (j==alleles[locus]+1-spare) {
					if (!spare) {
						printf("alleles %d j = %d  val = %d\n",alleles[locus],j,m[i][2*locus]);
						myerror("error in recode");
					} else
						m[i][2*locus] =j ;
				}
			}
		}
	}
}
/*******************************************************************/
int **count_alleles(int **d1,int nd1,int **d2, int nd2, int **d3, 
					int nd3, int **d4, int nd4, int loci, int *alleles)
{
	int *itemp,locus,i,j,count,totn,max_alleles,**alabels;
	
	max_alleles=0;
	alabels=(int **)MALLOC((loci+1)*sizeof(int*));
	if (!alabels) myerror("error allocating memory");
	totn=nd1+nd2+nd3+nd4;
	itemp=nrivector(1,2*totn);
	for (locus=1;locus<=loci;locus++) {
		count=1;
		for (i=1;i<=nd1;i++) {
			itemp[count++]=d1[i][2*locus-1];itemp[count++]=d1[i][2*locus];
		}
		for (i=1;i<=nd2;i++) {
			itemp[count++]=d2[i][2*locus-1];itemp[count++]=d2[i][2*locus];
		}
		for (i=1;i<=nd3;i++) {
			itemp[count++]=d3[i][2*locus-1];itemp[count++]=d3[i][2*locus];
		} for (i=1;i<=nd4;i++) {
			itemp[count++]=d4[i][2*locus-1];itemp[count++]=d4[i][2*locus];
		}
		ihpsort(2*totn,itemp);
		if (itemp[1]==-1) alleles[locus]=0;
		else alleles[locus]=1;
		for (i=2;i<=2*totn;i++) {
			if (itemp[i]!=itemp[i-1]) alleles[locus]+=1;
		}
		alabels[locus]=nrivector(1,alleles[locus]);
		for (j=1;;j++) if (itemp[j]!=-1) break;
		alabels[locus][1]=itemp[j];
		count=1;
		for (i=j+1;i<=2*totn;i++) {
			if (itemp[i]!=itemp[i-1]) 
				alabels[locus][++count]=itemp[i];
		}	
		if (alleles[locus]>max_alleles) max_alleles=alleles[locus];
	}
	free_ivector(itemp,1);
	recode(d1,nd1,0,loci,alleles,alabels); // no spares here
	recode(d2,nd2,0,loci,alleles,alabels);
	recode(d3,nd3,0,loci,alleles,alabels);
	recode(d4,nd4,0,loci,alleles,alabels);
	return alabels;
}
/*******************************************************************/
/*   This version of count alleles bins all those that are not in 
sample from the string - ie it just gets those in the matrix d1 
if there are any others in d2 it just puts them all in one bin     */
/*******************************************************************/
int  **count_allelesb(int **d1,int nd1,int **d2,int nd2,int **d3, int nd3, int **b, int nb, 
					int loci, int *alleles, int format)
{
	int *itemp,locus,i,j,count,ntot,**alabels;
	

	alabels=(int **)MALLOC((loci+1)*sizeof(int*));
	if (!alabels) myerror("error allocating memory");
	ntot=nd1+nd2+nd3;
	itemp=nrivector(1,2*ntot);
	if (format=='N'||format=='n') 
		printf("Alleles Used (9999 represents those in population but not sample)\n");
	else 
		printf("Alleles Used (@ represents those in population but not sample)\n");
	for (locus=1;locus<=loci;locus++) {
		printf("locus %d\t|",locus);
		count=1;
		for (i=1;i<=nd1;i++) {
			itemp[count++]=d1[i][2*locus-1];itemp[count++]=d1[i][2*locus];
		}
		for (i=1;i<=nd2;i++) {
			itemp[count++]=d2[i][2*locus-1];itemp[count++]=d2[i][2*locus];
		}
		for (i=1;i<=nd3;i++) {
			itemp[count++]=d3[i][2*locus-1];itemp[count++]=d3[i][2*locus];
		}
		ihpsort(2*ntot,itemp);
		if (itemp[1]==-1) alleles[locus]=0;
		else alleles[locus]=1;
		for (i=2;i<=2*ntot;i++) {
			if (itemp[i]!=itemp[i-1]) alleles[locus]+=1;
		}
		alleles[locus]+=1; /* add one for alleles not in sample */
		alabels[locus]=nrivector(1,alleles[locus]);
		for (j=1;;j++) if (itemp[j]!=-1) break;
		alabels[locus][1]=itemp[j];
		count=1;
		for (i=j+1;i<=2*ntot;i++) {
			if (itemp[i]!=itemp[i-1]) 
				alabels[locus][++count]=itemp[i];
		}
		alabels[locus][++count] = 9999;  /* the bin*/

		if (format=='N'||format=='n') {
			for (i=1;i<=count;i++) 
				printf("%d\t",alabels[locus][i]);
		} else 	{
			for (i=1;i<=count;i++) 
				printf("%c\t",(alabels[locus][i]<9999)?alabels[locus][i]:'@');	
		}	
		printf("\n");
		if (count!=alleles[locus]) myerror("error 1 in count_alleles");
	}
	
	free_ivector(itemp,1);
	recode(d1,nd1,0,loci,alleles,alabels);  //none spare - has to fit exactly
	recode(d2,nd2,0,loci,alleles,alabels);
	recode(d3,nd3,0,loci,alleles,alabels);
	recode(b,nb,1,loci,alleles,alabels);    // there is a spare here!!

	return alabels;
}
/*******************************************************************/
double **calculate_rho(int **data, int n, double before, int loci,int *alleles)
{
	int locus,i,max_alleles;
	double **tmp;
	max_alleles=imax(alleles,loci);
	tmp=nrdmatrix(1,loci,1,max_alleles);
	if (!data) {
		for (locus=1;locus<=loci;locus++) 
			for (i=1;i<=alleles[locus];i++) tmp[locus][i]=before;
	} else {
		for (locus=1;locus<=loci;locus++) {
			for (i=1;i<=alleles[locus];i++) tmp[locus][i]=before;
			for (i=1;i<=n;i++) {
				tmp[locus][data[i][2*locus-1]]+=1.0;
				tmp[locus][data[i][2*locus]]+=1.0;
			}
		}
	}
	return tmp;
}
/*******************************************************************/
double **calculate_pi(int **data, int n, int loci, int *alleles, int max_alleles)
{
	int locus,i;
	double **tmp;
	

	tmp=nrdmatrix(1,loci,1,max_alleles);
	if (!data) {
		for (locus=1;locus<=loci;locus++) 
			for (i=1;i<=alleles[locus];i++) tmp[locus][i]=1./(double)alleles[locus];
	} else {
		for (locus=1;locus<=loci;locus++) {
			for (i=1;i<=alleles[locus];i++) tmp[locus][i]=0.0;
			for (i=1;i<=n;i++) {
				tmp[locus][data[i][2*locus-1]]+=1.0;
				tmp[locus][data[i][2*locus]]+=1.0;
			}
			for (i=1;i<=alleles[locus];i++) tmp[locus][i]/=2.*n;
		}
	}
	return tmp;
}
/**********************************************************************/
data read_data(parameters *p)
{
	data d;
	int itmp;
	
	d.Y=readcharintegermatrix(openinputfile(p->datafile),&d.samples,&d.loci);
	d.loci/=2;
	if (p->useloci||p->usesamp) {
		d.Y = subsetdata(d.Y,d.samples,p->usesamp,d.loci,p->useloci,1);
		if (p->usesamp) d.samples=p->usesamp[0];
		if (p->useloci) d.loci=p->useloci[0];
	}
	printf("Read in genotypes for %d individuals at %d loci\n",d.samples,d.loci);
	d.alleles=nrivector(1,d.loci);

	if (p->knownfathers) {
		d.knownfathers=readintegervector(openinputfile(p->knownfathers),&itmp);
		d.knownfathers[0]=itmp;
		if (itmp!=d.samples) {
			myerror("error, wrongfathers wrong length");
		}
	}
	if (p->knownmothers) {
		d.knownmothers=readintegervector(openinputfile(p->knownmothers),&itmp);
		d.knownmothers[0]=itmp;
		if (itmp!=d.samples) {
			myerror("error, wrongmothers wrong length");
		}
	}
    if (p->freqfile) {
		d.background_freq=readcharintegermatrix(openinputfile(p->freqfile),&d.freqsamples,&itmp);
		if (p->useloci) 
			d.background_freq = subsetdata(d.background_freq,
			d.freqsamples,NULL,itmp/2,p->useloci,1);
	} else {
		d.background_freq=NULL;
		d.freqsamples=0;
	}

	if (p->fatherfile){
		d.fixed_males=readcharintegermatrix(openinputfile(p->fatherfile)
			,&d.nfixedfathers,&itmp);
		if (p->useloci) d.fixed_males= subsetdata(d.fixed_males,d.nfixedfathers,NULL,itmp,p->useloci,1);
	} else {
		d.fixed_males=NULL;
		d.nfixedfathers=0;
	}

	if (p->motherfile){
		d.fixed_females=readcharintegermatrix(openinputfile(p->motherfile)
			,&d.nfixedmothers,&itmp);
		if (p->useloci) d.fixed_females= subsetdata(d.fixed_females,d.nfixedmothers
			,NULL,itmp,p->useloci,1);
	} else {
		d.fixed_females=NULL;
		d.nfixedmothers=0;
	}
	
	if (p->freqmodel) {
		d.allele_labels=count_alleles(d.Y,d.samples,d.fixed_males,d.nfixedfathers,d.fixed_females
			,d.nfixedmothers,d.background_freq,d.freqsamples,d.loci,d.alleles);
		d.max_alleles=imax(d.alleles,d.loci);
		d.rho=calculate_pi(d.background_freq,d.freqsamples,d.loci,d.alleles,d.max_alleles);
	} else {
		d.allele_labels=count_allelesb(d.Y,d.samples,d.fixed_males,d.nfixedfathers,
			d.fixed_females,d.nfixedmothers,d.background_freq,d.freqsamples,d.loci,d.alleles,p->format);
		d.max_alleles=imax(d.alleles,d.loci);
		d.rho=calculate_rho(d.background_freq,d.freqsamples,1.0,d.loci,d.alleles);
	}

	Y=d.Y;
	alleles=d.alleles;
	loci=d.loci;
	max_alleles=d.max_alleles;
	freqmodel=p->freqmodel;
	allele_labels=d.allele_labels;
	rho=d.rho;
	samples=d.samples;

	
	return d;
}
/*******************************************************************/
/* a utility function for sort_genotypes below                     */
int compare(const void *g1,const void *g2)
{
	int l,*gen1,*gen2;
	gen1=*(int**)g1;
	gen2=*(int**)g2;
	for (l=1;l<=loci;l++) {
		if (gen1[2*l-1]<gen2[2*l-1]) return -1;
		else if (gen1[2*l-1]>gen2[2*l-1]) return 1;
		if (gen1[2*l]<gen2[2*l]) return -1;
		else if (gen1[2*l]>gen2[2*l]) return 1;
	}
	return 0;
}
/*******************************************************************/
void sort_genotypes(int **x, int n, int loci)
{
	int i,l,itmp;

	for (i=1;i<=n;i++) {
		for (l=1;l<=loci;l++) {
			if (x[i][2*l-1]>x[i][2*l]) {
				itmp=x[i][2*l];
				x[i][2*l]=x[i][2*l-1];
				x[i][2*l-1]=itmp;
			}
		}
	}
	qsort((void *)(x+1),(size_t)n,sizeof(int*),compare);
}
/*******************************************************************/
void print_geno(FILE *out, char *message ,int *gen,char format)
{ 
	int i,a1,a2;
	fprintf(out,"%s",message);
	if (format=='N'||format=='n') {
		for (i=1;i<=loci;i++) {
			fprintf(out,"%d %d\t",allele_labels[i][gen[2*i-1]],
				allele_labels[i][gen[2*i]]);
		}
	} else {
		for (i=1;i<=loci;i++) {
			a1=allele_labels[i][gen[2*i-1]];a2=allele_labels[i][gen[2*i]];
			a1=(a1<9999)?a1:'@';
			a2=(a2<9999)?a2:'@';
			a1=(a1>0)?a1:'?';
			a2=(a2>0)?a2:'?';
			fprintf(out,"%c %c\t",a1,a2);
		}
	}
	fprintf(out,"\n");
}
/*****************************************************************************************/
