/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "nrutil.h"
#include "util.h"
#include "random.h"
#include "mcmc_paternity.h"
#include "patlookup.h"
#include "patutil.h"
#include "prior.h"
#include "myio.h"
#include "randparent.h"

/*******************************************************************/
/*******************************************************************/
extern int loci,*alleles,samples,**Y,freqmodel,max_alleles;
extern double **rho;
/**********************************************************************/
/**********************************************************************/
void gibbs_foffspring(chain *c)
{
#ifdef CHATTER
	printf("gibbs_foffspring\n");
#endif
    c->fathers.gibbsparentaloffspring(&c->fathers,&c->mothers,c->temper,c->muindex);//,c->proploc);
}
/*******************************************************************/
void gibbs_moffspring(chain *c)
{
#ifdef CHATTER
	printf("gibbs_moffspring\n");
#endif
	c->mothers.gibbsparentaloffspring(&c->mothers,&c->fathers,c->temper,c->muindex);//,c->proploc);
}
/*******************************************************************/
void gibbs_offspring(chain *c)
{
#ifdef CHATTER
	printf("gibbs_offspring\n");
#endif
	if (c->fathers.n==1&&c->mothers.n==1) return;
	
	if (c->fathers.fixed==0&&c->mothers.fixed==0)  
		c->sampleallparentaloffspring(&c->fathers,&c->mothers,c->temper,c->muindex);//,c->proploc);
	return;
}
/*******************************************************************/
void gibbs_mothers(chain *c)
{
#ifdef CHATTER
	printf("gibbs_mothers\n");
#endif
	if (c->mothers.fixed==c->mothers.n) return;
	gibbsparentalgenotypes(&c->mothers,&c->fathers,c->temper,c->temper3,
		c->muindex);//,c->proploc);
	return;
}
/*******************************************************************/
void gibbs_fathers(chain *c)
{ 
#ifdef CHATTER
	printf("gibbs_fathers\n");
#endif
	if (c->fathers.fixed==c->fathers.n) return;
	gibbsparentalgenotypes(&c->fathers,&c->mothers,c->temper,c->temper3,
		c->muindex);//,c->proploc);
	return;
}
/*******************************************************************/
int metro_mu(chain *c,long *n)
{
	double newmu,lpdiff=0.0,x;//,l[2];
	int i,j;
#ifdef CHATTER
	printf("metro_mu\n");
#endif
	*n+=1;
	if (c->muprior.prtype==CONSTPRIOR) {
		return 0;
	}
	
	newmu=exp(log(c->mu)+ranDum()-0.5);
	gettrylookup(newmu);
	for (j=1;j<=c->loci;j++) {	
		for (i=1;i<=samples;i++) 
			lpdiff+=log_lookup_diff(Y[i]+2*j-2,
			c->fathers.genotype[c->fathers.parentage[i]]+2*j-2,
			c->mothers.genotype[c->mothers.parentage[i]]+2*j-2,c->muindex);//*c->proploc[j];
	}
	
	x = exp(lpdiff/c->temper+log(newmu)-log(c->mu)+(
		log_prior(&newmu,c->muprior)-log_prior(&c->mu,c->muprior)
		)/c->temper3);

	if (x>1.0||ranDum()<x) {
		c->mu=newmu;
		swap_tables(c->muindex);
		return 1;
	} else {
		return 0;
	}
}
/*******************************************************************/
int metro_albe(parents *p, double temper)
{
	double newalbe,lpdiff=0.0,x;
	
	newalbe=exp(log(p->alpha)+ranDum()-0.5);
	
	lpdiff = ll_off(p->offspring,p->n,newalbe,&p->off) - 
		ll_off(p->offspring,p->n,p->alpha,&p->off);
	
	x = exp(lpdiff +log(newalbe)-log(p->alpha)+
		(log_prior(&newalbe,p->off.alphaprior)-log_prior(&p->alpha,p->off.alphaprior)
		)/temper
		);
	
	if (x>1.0||ranDum()<x) {
		p->alpha=newalbe;
		return 1;
	} else {
		return 0;
	}
}
/*******************************************************************/
int metro_alpha(chain *c, long  *n)
{
#ifdef CHATTER
	printf("metro_alpha\n");
#endif
	*n+=1;
	return metro_albe(&c->fathers,c->temper3);
}
/*******************************************************************/
int metro_beta(chain *c, long *n)
{
#ifdef CHATTER
	printf("metro_beta\n");
#endif
	*n+=1;
	return metro_albe(&c->mothers,c->temper3);
}
/*******************************************************************/
double priorproploc(double *pl)
{
	double *a,lp;
	int i;
	
	a=copy_dvector(pl,1,loci);
	for (i=1;i<=loci;i++) 
		a[i] /= ((double)loci*pl[0]);
	
	lp = lddirichlet(a,100.,loci);
	free_dvector(a,1);
	return lp;
}
/******************************************************/
/*   Metropolis function for changing the proportion  */
/*   of the likelihood that is associated with each   */
/*   locus for tempered chains                        */
/******************************************************/ 
/*int metro_proploc(chain *c, long *n)
{
	int wh[2];
    double x,*newproploc,sm;
#ifdef CHATTER
	printf("metro_proploc\n");
#endif	
	if (c->proploc[0]<=0.0) return 0;  //the untempered chain
	if (c->loci==1) return 0;
	
	*n+=1;	
	newproploc=copy_dvector(c->proploc,0,c->loci);
	sample2int(wh,1,c->loci);
	sm=c->proploc[wh[0]]+c->proploc[wh[1]];
    if (sm>=2.0||sm<=0.0) {
		mywarning("problem: sm > 2 or < 0 in metro_proploc");
		free_dvector(newproploc,0);
		return 1;
	} 
	if (sm>1.0) {		
		newproploc[wh[0]]=(sm-1.0) + ranDum()*(2.-sm);
	} else {
		newproploc[wh[0]]=ranDum()*sm;
	}
	newproploc[wh[1]]=sm-newproploc[wh[0]];
	
	x = exp(priorproploc(newproploc)-priorproploc(c->proploc)+
		log_posterior(c,c->temper,c->temper3,newproploc)-
		log_posterior(c,c->temper,c->temper3,c->proploc)
		);
	if (x>1.0||ranDum()<x) {
		c->proploc[wh[0]]=newproploc[wh[0]];
		c->proploc[wh[1]]=newproploc[wh[1]];
		free_dvector(newproploc,0);
		return 1;
	} else {
		free_dvector(newproploc,0);
		return 0;
	}
}*/
/******************************************************/
/*  function for performing a MCMC tempering step
    that is swapping the states between states with 
	different tempering values                        */
/******************************************************/ 
int metro_temper(model *m,int indx, long *n)
{
	double x,tmp;//,*dvtmp;
	chain *c1,*c2;
	int itmp,i;
#ifdef CHATTER
	printf("metro_temper\n");
#endif
	c1=&m->c[1];   //set this to avoid wqarning messages !!
	for (i=2;i<=m->tempering+1;i++) {
		if (m->c[i].temperindex==indx) {
			c1 = &m->c[i];
			break;
		}
	}
	c2=&m->c[1];
	for (i=2;i<=m->tempering+1;i++) {
		if (m->c[i].temperindex==indx+1) {
			c2=&m->c[i];
			break;
		}
	}
	
	*n+=1;
	x = exp(
		log_posterior(c2,c1->temper,c1->temper3)//,c1->proploc)
		+log_posterior(c1,c2->temper,c2->temper3)-//,c2->proploc)-
		log_posterior(c1,c1->temper,c1->temper3)//,c1->proploc)
		-log_posterior(c2,c2->temper,c2->temper3)//,c2->proploc)
		);
	if (x>1.0||ranDum()<x) {
		tmp=c1->temper;
		c1->temper=c2->temper;
		c2->temper=tmp;
		tmp=c1->temper3;
		c1->temper3=c2->temper3;
		c2->temper3=tmp;
		itmp=c1->temperindex;
		c1->temperindex=c2->temperindex;
		c2->temperindex=itmp;
//		dvtmp=c1->proploc;
//		c1->proploc=c2->proploc;
//		c2->proploc=dvtmp;
		return 1;
	}
	return 0;
}
/******************************************************************/
/*  try to get a completely different sample - from the sample_chain
    function then perform a MCMC step - the idea is to get faster 
	by using an independence sampler, at least for the chains 
	that are tempered - the last chain out of the tempering is 
    from an independence sampler and we just sample from it 
    when needed                                                   */
/******************************************************************/
int metro_newsample(model *m,int indx, long *n)
{
	double x,lpfor,lpback,lpdiff;
	int i;
	chain *c1,*cnew;

	if (m->c[1].fathers.known||m->c[1].mothers.known) {
		myerror("error:  independence sampling is "
			"not yet written when any offspring relationships\n"
			"are known");
	}

#ifdef CHATTER
	printf("metro_newsample\n");
#endif
	cnew=&m->c[0];
	c1=&m->c[1];   //set to avoid warning messages
	for (i=2;i<=m->tempering+1;i++) {
		if (m->c[i].temperindex==indx) {
			c1=&m->c[i];
			break;
		}
	}
	
//	static FILE *xfile;
//	 static int open=0;

//	if (!open) {
//		xfile=openoutputfile("xfile.out");
//		open=1;
//	}

	//printf("here");

	if (c1->mothers.n!=1) return 0;
	*n+=1;
	// copy things that don't change 
	cnew->muindex=c1->muindex;
	cnew->mu=c1->mu;
//	for (i=1;i<=c1->loci;i++) cnew->proploc[i]=1.0;
	cnew->temper=c1->temper;
	cnew->temper3=c1->temper;
	cnew->temperindex=c1->temperindex;

	lpfor = sample_chainb(cnew);//,cnew->proploc);
	lpback = lpsample_chainb(c1);//,cnew->proploc);

	lpdiff=log_posterior(cnew,cnew->temper,cnew->temper3/*,c1->proploc*/) - 
		log_posterior(c1,c1->temper,c1->temper3/*,c1->proploc*/);

//	fprintf(xfile,"%g\n",lpdiff-lpfor+lpback);
	x=exp(lpdiff-lpfor+lpback);
//	printf("new chain  %d %d muts pold %g pnew %g   x = %g\n",cnew->fathers.n,count_muts(&cnew->fathers,&cnew->mothers),
//log_posterior(c1,c1->temper,c1->temper3,c1->proploc),log_posterior(cnew,cnew->temper,cnew->temper3,c1->proploc),x 
//	);
	
//	printf("lpdiff = %g, lpfor = %g, lpback = %g  x = %g\n",lpdiff,lpfor,lpback,x);

	if (x>1.||ranDum()<x) {
		copy_parentsinplace(&c1->fathers,&cnew->fathers);  
        copy_parentsinplace(&c1->mothers,&cnew->mothers);
		return 1;
	} 
	else return 0;
}
/*************************************************************/
/*   Attempts to swap the male and female genotypes around   
     can only be used if there are no male or female genotypes 
	 fixed    */
/*************************************************************/
int metro_swapmalesfemales(chain *c)
{
	double x;
#ifdef CHATTER
	printf("swapmalesfemales\n");
#endif
	/*  First there are rather a lot of special cases to consider */

	if (c->fathers.fixed>0) return 0;
	if (c->mothers.fixed>0) return 0;
	if (c->fathers.n>c->mothers.range[1]||c->mothers.n>c->fathers.range[1])
		return 0;
	if (c->fathers.n<c->mothers.range[0]||c->mothers.n<c->fathers.range[0])
		return 0;

/*	if (c->fathers.nprior.prtype==CONSTPRIOR&&
		c->mothers.n!=(int)(c->fathers.nprior.par[0]+0.5)) return 0;
	if (c->mothers.nprior.prtype==CONSTPRIOR&&
		c->fathers.n!=(int)(c->mothers.nprior.par[0]+0.5)) return 0;*/

	x = exp(
		lpost_off(c->fathers.offspring,c->fathers.n,c->mothers.alpha,&c->mothers.off,c->temper3)
		+lpost_off(c->mothers.offspring,c->mothers.n,c->fathers.alpha,&c->fathers.off,c->temper3)
		-lpost_off(c->fathers.offspring,c->fathers.n,c->fathers.alpha,&c->fathers.off,c->temper3)
		-lpost_off(c->mothers.offspring,c->mothers.n,c->mothers.alpha,&c->mothers.off,c->temper3)
		);

	if (x>1.0||ranDum()<x) {
		swapmalesfemales(c);
		return 1;
	}
	return 0;
}
/*************************************************************/
/*  A metropolis step that merges the offspring of 
    a pair of parents (or splits the offspring) and then 
    resamples the genotypes. This version works when there
	are no fixed genotypes                                   */
/*************************************************************/
int metro_splitjoin(parents *ch, parents *st,double temper,
					double temper3, int index)//, double *proploc)
{
    int i,j,oldn,*oldpar,**oldgeno,*oldoff,locus;
	double x,lpfor,lpback,lpdiff=0.0;

#ifdef CHECK
#ifdef CHATTER
	printf("splitjoin \n");fflush(stdout);
#endif
	check_off(ch->parentage,ch->offspring,ch->n,ch->off.model);
	check_off(st->parentage,st->offspring,st->n,ch->off.model);
#endif
	/*  first collect the old values        */
	oldpar=copy_ivector(ch->parentage,1,samples);
	oldgeno=copy_imatrix(ch->genotype,1,ch->n,1,2*loci);
	oldoff=copy_ivector(ch->offspring,1,ch->n);
	oldn=ch->n;
	lpback= lpselectedparentalgenotypes(ch,st,temper,temper3,index);//,proploc);
	
	if (ch->n>ch->range[0]&&ranDum()<0.5||ch->n==ch->range[1]) { 
#ifdef CHATTER
		printf("join \n");fflush(stdout);
#endif
		/* join two parents together */			
		lpfor = ch->joinparentage(ch,&lpback);
		if (ch->n<ch->range[1]) lpfor -= M_LN2;
		ch->n-=1;
		if (ch->n>ch->range[0]) lpback -= M_LN2;
	} else { /*  split  */
#ifdef CHATTER
		printf("split \n");fflush(stdout);
#endif
		lpfor = ch->splitparentage(ch,&lpback);
		if (ch->n>ch->range[0]) lpfor -= M_LN2;
		ch->n+=1;
	    if (ch->n<ch->range[1]) lpback -= M_LN2;
	}
	
	lpfor += gibbsparentalgenotypes(ch,st,temper,temper3,index);//,proploc);

	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=samples;i++) {
			lpdiff += (
				log_lookup_prob(Y[i]+2*locus-2,ch->genotype[ch->parentage[i]]+2*locus-2,
				st->genotype[st->parentage[i]]+2*locus-2,index) -
				log_lookup_prob(Y[i]+2*locus-2,oldgeno[oldpar[i]]+2*locus-2,
				st->genotype[st->parentage[i]]+2*locus-2,index)
				);//*proploc[locus];
		}
	}

	lpdiff/=temper;
	lpdiff += lpost_off(ch->offspring,ch->n,ch->alpha,&ch->off,temper3);
	lpdiff -= lpost_off(oldoff,oldn,ch->alpha,&ch->off,temper3);
	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=ch->n;i++) {
			if (freqmodel) 
				lpdiff+=log_multinomial3temper(
				ch->genotype[i]+2*locus-2,rho[locus],alleles[locus],2,temper3/*proploc[locus]*/);
			else 
				lpdiff+=log_dmulti_dirichletctemper(
				ch->genotype[i]+2*locus-2,rho[locus],alleles[locus],2,temper3/*proploc[locus]*/);
		}
		for (i=1;i<=oldn;i++) {
			if (freqmodel) 
				lpdiff-=log_multinomial3temper(
				oldgeno[i]+2*locus-2,rho[locus],alleles[locus],2,temper3/*proploc[locus]*/);
			else 
				lpdiff-=log_dmulti_dirichletctemper(
				oldgeno[i]+2*locus-2,rho[locus],alleles[locus],2,temper3/*proploc[locus]*/);
		}
	}	
	x = exp(lpdiff-lpfor+lpback);
	if (x>1.0||ranDum()<x) {
		free_imatrix(oldgeno,1,1);
		free_ivector(oldpar,1);
		free_ivector(oldoff,1);
		return 1;
	} else {
		ch->n=oldn;
		for (i=1;i<=oldn;i++) {
			ch->offspring[i]=oldoff[i];
			for (j=1;j<=2*loci;j++) ch->genotype[i][j]=oldgeno[i][j];
		}
		for (i=1;i<=samples;i++) ch->parentage[i]=oldpar[i];
		free_ivector(oldoff,1);
		free_imatrix(oldgeno,1,1);
		free_ivector(oldpar,1);
		return 0;
	}
}
/*********************************************************************/
int metro_rjfather(chain *c,long *n)
{
#ifdef CHATTER
		printf("metro_rjfather  ");fflush(stdout);
#endif
	if (c->fathers.fixed==c->fathers.range[1]) return 0;
	
	*n+=1;
	if (c->fathers.range[0]==c->fathers.range[1]) {
		if (c->fathers.n!=c->fathers.range[0]) 
			myerror("incorrect number of males in metro_rjfather");
		return 1;
	}
	if (ranDum()<0.04) {
		return metro_swapmalesfemales(c);
	} else 
		return metro_splitjoin(&c->fathers,&c->mothers,c->temper,
		c->temper3,c->muindex);//,c->proploc);
}
/*************************************************************/
int metro_rjmother(chain *c,long *n)
{
#ifdef CHATTER
	printf("metro_rjmother\n");fflush(stdout);
#endif

	if (c->mothers.fixed==c->mothers.range[1]) return 0;

	*n+=1;
	if (c->mothers.range[0]==c->mothers.range[1]) {
		if (c->mothers.range[0]!=c->mothers.n) {
			printf("have %d females\n",c->mothers.n);
			myerror("incorrect number of females in metro_mother");
		}
		return 0;
	}	

	if (ranDum()<0.04) 
		return metro_swapmalesfemales(c);
	else  
	    return metro_splitjoin(&c->mothers,&c->fathers,c->temper,
			c->temper3,c->muindex);//,c->proploc);	
}
/*************************************************************/
/*  Note that this always assumes that both alleles at a locus
    Are missing*/
void gibbs_missing(chain *c) 
{
	int i,par,loc;

	for (i=1;i<=c->fathers.nmissing;i++) {
		par=c->fathers.mssp[i];
		loc=c->fathers.mssl[i];
        gibbsonelocus(c->fathers.genotype[par]+2*loc-2,
			&c->mothers,c->temper,c->temper3,c->muindex,c->fathers.parentage,
			par,loc,alleles[loc]);
	} 
	for (i=1;i<=c->mothers.nmissing;i++) {
		par=c->mothers.mssp[i];
		loc=c->mothers.mssl[i];
        gibbsonelocus(c->mothers.genotype[par]+2*loc-2,
			&c->fathers,c->temper,c->temper3,c->muindex,c->mothers.parentage,
			par,loc,alleles[loc]);
	}
}
/*************************************************************/


