/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <string.h>
#include "model.h"
#include "mcmc_paternity.h"
#include "util.h"
#include "nrutil.h"
#include "patlookup.h"
#include "chain.h"

/**********************************************************/
/*  A utility function to set up the different metropolis
    and gibbs functions that will be used   - also gives
names so that the correct metroplis proportions will be
given  - will also allow the removal of a few more checks */
/**********************************************************/
int choose_gibbsmets(model *m)
{
	m->ngibbs=m->nmets=0;
		
/*  first look at fathers  */	
	if (m->c[1].fathers.fixed) {
		if (m->c[1].fathers.range[0]==m->c[1].fathers.range[1]) {
			if (m->c[1].fathers.fixed!=m->c[1].fathers.range[1])
				m->gibbsfuncs[m->ngibbs++] = gibbs_fathers;
		}
		if (m->c[1].fathers.known) {
			if (m->c[1].fathers.known[0]!=m->dat.samples)
				m->gibbsfuncs[m->ngibbs++] = gibbs_foffspring;
		} else m->gibbsfuncs[m->ngibbs++] = gibbs_foffspring;
	} else {
		m->gibbsfuncs[m->ngibbs++] = gibbs_fathers;
		m->gibbsfuncs[m->ngibbs++] = gibbs_foffspring;
	}
	if (m->c[1].fathers.range[0]!=m->c[1].fathers.range[1]) {
		strcpy(m->metlabel[m->nmets],"rjfather");
		m->metrofuncs[m->nmets++] = metro_rjfather;
	}
	if (m->c[1].mothers.fixed) {
		if (m->c[1].mothers.range[0]==m->c[1].mothers.range[1]) {
			if (m->c[1].mothers.fixed!=m->c[1].mothers.range[0])
				m->gibbsfuncs[m->ngibbs++] = gibbs_mothers;
		}
		if (m->c[1].mothers.known) {
			if (m->c[1].mothers.known[0]!=m->dat.samples)
				m->gibbsfuncs[m->ngibbs++] = gibbs_moffspring;
		} else m->gibbsfuncs[m->ngibbs++] = gibbs_moffspring;
	} else {
		m->gibbsfuncs[m->ngibbs++] = gibbs_mothers;
		m->gibbsfuncs[m->ngibbs++] = gibbs_moffspring;
	}
	if (m->c[1].mothers.range[0]!=m->c[1].mothers.range[1]) {
		strcpy(m->metlabel[m->nmets],"rjmother");
		m->metrofuncs[m->nmets++] = metro_rjmother;
	}
	
	if (!m->c[1].fathers.known&&!m->c[1].mothers.known)
		m->gibbsfuncs[m->ngibbs++] = gibbs_offspring;
	
	if (m->c[1].muprior.prtype!=CONSTPRIOR) {
		strcpy(m->metlabel[m->nmets],"mu");
		m->metrofuncs[m->nmets++]=metro_mu;
	}
	if (m->c[1].fathers.off.model>1&&m->c[1].fathers.off.alphaprior.prtype!=CONSTPRIOR) {
		strcpy(m->metlabel[m->nmets],"alpha");
		m->metrofuncs[m->nmets++]=metro_alpha;
	}
	if (m->c[1].mothers.off.model>1&&
		m->c[1].mothers.off.alphaprior.prtype!=CONSTPRIOR) {		
		strcpy(m->metlabel[m->nmets],"beta");
		m->metrofuncs[m->nmets++]=metro_beta;
	}
	if (m->c[1].fathers.nmissing>0||m->c[1].mothers.nmissing>0) {
		m->gibbsfuncs[m->ngibbs++]=gibbs_missing;
		gibbs_missing(&m->c[1]);
	}
//	if (m->tempering) {
//		strcpy(m->metlabel[m->nmets],"prop");
//		m->metrofuncs[m->nmets++]=metro_proploc;
//	}
	m->success=nrlmatrix0(1,m->tempering+1,0,6);
	m->attempt=nrlmatrix0(1,m->tempering+1,0,6);
	return m->ngibbs+m->nmets+(m->tempering>0||m->param.independence>0);
}
/**********************************************************/
/*  A pair of functions to print the metroplis proportions
  and report on how the algorithms have performed         */
/**********************************************************/
void printmetrorates(FILE *OUT, long *pr, long *n,int ch, int nm,char ml[4][10])
{
	int i;
	if (nm==0) return;
	if (ch==0) 
		fprintf(OUT,"acceptance rates for MCMC steps\n");
	else 
		fprintf(OUT,"acceptance rates for MCMC steps for chain %d\n",ch);
	for (i=0;i<nm;i++) 
		fprintf(OUT,"%s %g\n",ml[i],(n[i]>0)?(double)pr[i]/(double)n[i]:0.0);	
}	
/*********************************************************************/
void printallrates(FILE *OUT,model *m)
{
	int k;

	printmetrorates(OUT,m->success[1],m->attempt[1],1,m->nmets,m->metlabel);
		fprintf(OUT,"\n");
	if (m->tempering) {	
		for (k=2;k<=m->tempering+1;k++) {
				printmetrorates(OUT,m->success[k],m->attempt[k],k,m->nmets,m->metlabel);
		}
	fprintf(OUT,"\n");
		fprintf(OUT,"\ntempering:\n");
		for (k=1;k<=m->tempering;k++) {
			fprintf(OUT,"%d<->%d %g\n",
				k,k+1,(m->attempt[k][m->nmets]>0)?
				(double)m->success[k][m->nmets]/
				(double)m->attempt[k][m->nmets]
				:0.0);
		}
	}
	if (m->param.independence) {
		fprintf(OUT,"independence sampler %g\n", 
			(m->attempt[m->tempering+1][m->nmets]>0)?
			(double)m->success[m->tempering+1][m->nmets]/
			(double)m->attempt[m->tempering+1][m->nmets]
			:0.0);
	}
	fprintf(OUT,"\n");
}
/**********************************************************/
void destroy_model(model *m)
{	
	int i;
	destroy_parameters(&m->param);
	free_lmatrix(m->success,1,0);
	free_lmatrix(m->attempt,1,0);
	for (i=1;i<=m->tempering+1;i++) {
		destroy_chain(&(m->c[i]));
	}
	if (m->param.independence) {
		destroy_chain(m->c);
	}
	FREE(m->c);
	destroy_data(&m->dat);
}
/**********************************************************/
void starting_chains(model *m)
{
	int i;

    /*  how many chains to run*/
	m->tempering=m->param.tempering;
	/* allocate memory for chains */
	m->c=(chain*)MALLOC((m->tempering+2)*sizeof(chain));
	if (!m->c) myerror("error allocating chain");

    /* get the first chain    */
	m->c[1]=starting_chain(&m->param,&m->dat);

	/* set up the lookup tables */
	setup_lookup(m->c[1].mu,m->tempering+1);
    /* if needed start the independence sampler  */
	if (m->param.independence) m->c[0]=copy_chain(&m->c[1]);
	for (i=1;i<=m->tempering;i++) {
		m->c[i+1]=copy_resample_chain(m->c+1);
		m->c[i+1].temper=1; 
		m->c[i+1].temper3=m->param.tempvals[i];
		m->c[i+1].temperindex=i+1;
		m->c[i+1].muindex=i+1;	
	}
}
/**********************************************************/
