#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "nrutil.h"
#include "util.h"
#include "myio.h"

/*************************************************************************/
/*  function to get a string from keyboard - skipping
	all blanks in the input - suitable for getting file
	names for example.  If nothing is entered then 
	def_val is used as a default                   */
/*************************************************************************/
char *getstringfromkeyboard(char *message, char *def_val)
{
	char *tmp;
	int i;
	int ch;

	tmp = (char*)malloc(200*sizeof(char));
	if (!tmp) myerror("error allocating memory in getstringfromkeyboard");
	printf("please enter %s, or press <enter> for default (%s)\n",
		message,def_val);
	for (i=0;;) {
		ch=skipblank(stdin);
		if (ch=='\n') {
			tmp[i++]='\0';
			break;
		} else tmp[i++]=(char)ch;
	}
	if (i==1) strcpy(tmp,def_val);
	return tmp;
}

/*************************************************************************/
/* A file with a collection of utilities for input and output            */
/*************************************************************************/
char *allocatestring(char *tocopy)
{
	int len;
	char *ret;
	
	if (tocopy !=NULL) {
		len=strlen(tocopy);
		ret=(char *)MALLOC((len+1)*sizeof(char));    
		if (!ret) myerror("error allocating string");
		strcpy(ret,tocopy);
		return ret;
	} else return NULL;
}
/*************************************************************************/
FILE *openoutputfile(char *filename)
{
	FILE *out;

	out  = fopen(filename,"w");
	if (!out) {
		printf("error opening output file %s\nexiting\n",filename);
		exit(0);
	}
	return out;
}
/*************************************************************************/
FILE *openappendfile(char *filename)
{
	FILE *out;
	out=  fopen(filename,"a");
	if (!out) {
		printf("error opening output file %s\nexiting\n",filename);
		exit(0);
	}
	return out;
}
/*************************************************************************/
FILE *openinputfile(char *filename)
{
	FILE *in;

	in = fopen(filename,"r");
	if (!in) {
		printf("error opening input file %s\nexiting\n",filename);
		exit(0);
	}
	return in;
}
/*************************************************************************/
FILE *openstemleafoutputfile(char *stem, char *leaf) 
{
	char filename[100];
	sprintf(filename,"%s.%s",stem,leaf);
	return openoutputfile(filename);
}
/*************************************************************************/
char *strdupcat(char *first,char *end)
{
	char *tmp;

	tmp = (char *)MALLOC((strlen(first)+strlen(end)+1)*sizeof(char));
	if (!tmp) myerror("error allocating memory in strdupcat");
	sprintf(tmp,"%s%s",first,end);
	return tmp;
}

#define FIRSTSIZE 1001
/*************************************************************************/
int *nextints(FILE *in,int len)
{
    int ch,*retvec,i;
    retvec=nrivector(1,len);
    ch = skipspace(in);
    if (ch!=':')
	nrerror("error integer should be preceded by a colon");
    for (i=1;i<=len;i++)
    	fscanf(in,"%d",retvec+i);
    return retvec;
}
/*************************************************************************/
void get_counter_filename(char *name, char *base, int count)
{
    sprintf(name,"%s.%d",base,count);
}
/****************************************************************/
char *readfromquotes(FILE *in,int *len)
{
	int count=0,ch,vecsize=FIRSTSIZE;
	char *data;
	
	ch = skipspace(in);
	if (ch!=39) {
		printf("error - should be a quote - ' is %c\n",ch);
		exit(0);
	}
	data = (char*)MALLOC(FIRSTSIZE*sizeof(char));
	if (!data) nrerror("error allocating memory in readfromquotes");
	for (count=0;;count++) {
		if (count==vecsize) {
			data = (char *)REALLOC(data,(vecsize+1000)*sizeof(char));
			if (!data) nrerror("error reallocating data in readfromquotes");
			vecsize +=1000;
		}
		ch = fgetc(in);
		if (ch==39) {
			data[count]='\0';
			break;
		}
		else data[count]=(char)ch;
		
	}
	data = (char *)REALLOC(data,(count+1)*sizeof(char));
    if (!data) nrerror("error reallocating data");
    *len = count+1;
	return data;
}
/****************************************************************/
double *readdoubleline(FILE *in, int *len)
{
    int count=0,vecsize=FIRSTSIZE;
    int ch;
    double *data;
    
	ch = skipspace(in);
	if (ch!=':') {
		ungetc(ch,in);
	}
	//    nrerror("error shoule be a colon first");

    data = (double*)MALLOC(FIRSTSIZE*sizeof(double));
    if (!data) nrerror("error allocating memory in readintegerline");
    for (;;) {
		ch = skipspace(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			if (count==vecsize) {
				data = (double *)REALLOC(data,(vecsize+1000)*sizeof(double));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
			}
			fscanf(in,"%lf",&(data[++count]));
		}
		else break;
    }
    data = (double *)REALLOC(data,(count+1)*sizeof(double));
    if (!data) nrerror("error reallocating data");
    *len = count;
    return data;
}
/************************************************************************/
void write_imatrix(FILE *out, int **mat, int sr, int er, int sc, int ec)
{
    int i,j;
    for (i=sr;i<=er;i++) {
		for (j=sc;j<=ec;j++)
			fprintf(out,"%d ",mat[i][j]);
		fprintf(out,"\n");
    }
    return;
}
/************************************************************************/
void write_raggedintegerarray(FILE *out, int **rag, int rows, int *cols)
{
    int i,j;
    for (i=1;i<=rows;i++) {
		for (j=1;j<=cols[i];j++)
			fprintf(out,"%d ",rag[i][j]);
		fprintf(out,"\n");
    }
    return;
}
/************************************************************************/
void write_raggeddoublearray(FILE *out, double **rag, int rows, int *cols)
{
    int i,j;
    for (i=1;i<=rows;i++) {
		for (j=1;j<=cols[i];j++)
			fprintf(out,"%g ",rag[i][j]);
		fprintf(out,"\n");
    }
    return;
}
/************************************************************************/
void write_dmatrix(FILE *out, double **mat, int sr, int er, int sc, int ec)
{
    int i,j;
    for (i=sr;i<=er;i++) {
		for (j=sc;j<=ec;j++)
			fprintf(out,"%g ",mat[i][j]);
		fprintf(out,"\n");
    }
    return;
}
/************************************************************************/
void write_dvector(FILE *out,char *sep, double *vec, int sr, int er)
{
    int i;
    for (i=sr;i<er;i++) {
			fprintf(out,"%g%s",vec[i],sep);    
	}
		fprintf(out,"%g\n",vec[er],sep); 
    return;
}
/************************************************************************/
void write_ivector(FILE *out, char *sep, int *vec, int sr, int er)
{
    int i;
    for (i=sr;i<er;i++) {
			fprintf(out,"%d%s",vec[i],sep);    
	}
	fprintf(out,"%d\n",vec[er]);
    return;
}
/*********************************************************************/
int get_doublesfrombrackets(FILE *in,double *par) 
{
    char ch;
    float f;
    
    ch=(char)skipspace(in);
    if (ch!='(') return 0;
    fscanf(in,"%f",&f);
    par[0]=f;
    ch=(char)skipspace(in);
    if (ch!=',') return 1;
    fscanf(in,"%f",&f);
    par[1]=f;
    ch=(char)skipspace(in);
    if (ch==')') return 2;
    else {
	printf("only two parameters so far\n");
	return 0;
    }
}
/*********************************************************************/
double get_doubleval(FILE *in) 
{
    char ch;
    double g;
    
    ch=(char)skipspace(in);
    if (ch!='<') return 0.0;
    fscanf(in,"%lg",&g);
	return g;
}
/*********************************************************************/
void get_doublevals(FILE *in,double *x, int *count) 
{
    char ch;
	
	*count=0;
    
    ch=(char)skipspace(in);
    if (ch!='<') return;
	for (;;) {
		*count +=1;
		fscanf(in,"%lg",x+(*count));
		ch=(char)skipspace(in);
		if (ch=='>') break;
	}
	return;
}
/*********************************************************************/
int skipline(FILE *in)
/*  a little function to read to the end of a line 
    to remove comments !!                            */
{
	char tmp[200];
	if (fgets(tmp,199,in)==NULL) {
		if (feof(in)) return EOF;
		else myerror("error reading in skipline");
	}
	return '\n';
}
/************************************************************************/
int skipspace(FILE * anyfile)
{
	static int      ch;
	
	for (;;) {
		ch = fgetc(anyfile);
		if (ch=='#') 
			ch = skipline(anyfile);
		if (!isspace(ch)) return (ch);
	}
}
/************************************************************************/
int skipblank(FILE * anyfile)
{
	static int      ch;

	for (;;) {
	    ch = fgetc(anyfile);
		if (ch=='#') 
			ch = skipline(anyfile);
	    if (ch=='\n') return (ch);
	    if (!isspace(ch)) return (ch);
	}
}
/*************************************************************************/
int nextint(FILE *in)
{
    int ch;
    ch = skipspace(in);
    if (ch!=':')
	nrerror("error integer should be preceded by a colon");
    fscanf(in,"%d",&ch);
    return ch;
}
/****************************************************************/
int *readintegerline(FILE *in)
{
    int *data,count=0,vecsize=FIRSTSIZE;
    int ch;
	ch = skipspace(in);
	if (ch!=':')
	    nrerror("error should be a colon first");

    data = (int*)MALLOC(FIRSTSIZE*sizeof(int));
    if (!data) nrerror("error allocating memory in readintegerline");
    for (;;) {
	ch = skipblank(in);
	if ((isdigit(ch))||(ch=='-')) {
	    ungetc(ch,in);
	    if (count==vecsize) {
		data = (int *)REALLOC(data,(vecsize+1000)*sizeof(int));
		if (!data) nrerror("error reallocating data");
		vecsize +=1000;
	    }
	    fscanf(in,"%d",&(data[++count]));
	}
	else break;
    }
    data = (int *)REALLOC(data,(count+1)*sizeof(int));
    if (!data) nrerror("error reallocating data");
    data[0] = count;
    return data;
}
/****************************************************************/
int *readcharintegerline(FILE *in)
{
    int *data,count=0,vecsize=FIRSTSIZE;
    int ch;
	ch = skipspace(in);
	if (ch!=':')
		nrerror("error should be a colon first");
	
    data = (int*)MALLOC(FIRSTSIZE*sizeof(int));
    if (!data) nrerror("error allocating memory in readintegerline");
    for (;;) {
		ch = skipspace(in);
		if ((isdigit(ch))||(ch=='-')) {
			ungetc(ch,in);
			fscanf(in,"%d",&(data[++count]));
		} else if (ch=='?') {
			data[++count]=-1;
		} else if (isalpha((int)ch)) {
			data[++count]=(int)ch;
		} else break;
		if (count==vecsize) {
			data = (int *)REALLOC(data,(vecsize+1000)*sizeof(int));
			if (!data) nrerror("error reallocating data");
			vecsize +=1000;
		}
    }
    data = (int *)REALLOC(data,(count+1)*sizeof(int));
    if (!data) nrerror("error reallocating data");
    data[0] = count;
    return data;
}
/************************************************************************/
char **read_characterarray(FILE *in,int *len)
{
	char **toret,space[]=" \t\n";
	int maxcount=10000;
	*len=1;


	toret= (char **)MALLOC(maxcount*sizeof(char*));
	if (!toret) myerror("error allocating memory in read_characterarray");
	toret[0] = (char *)MALLOC((1001)*sizeof(char));
	if (!toret[0]) myerror("error alocating memory");

	if (fgets(toret[0],1000,in)==NULL) myerror("error reading placenames");
	toret[0]=(char *)REALLOC(toret[0],(1+strlen(toret[0]))*sizeof(char));
	if (!toret[0]) myerror("error reallocating memory");

	toret[*len]=strtok(toret[0],space);
    while( toret[*len] != NULL ){
		*len+=1;
		toret[*len] = strtok(NULL,space);
	}
	*len-=1;
	
	toret = (char **)REALLOC(toret,(*len+1)*sizeof(char *));
	if (!toret) myerror("error reallocating memory in read_characterarray");

	return toret;
}
/************************************************************************/
char *readstring(FILE *in)
{
	char *temp;
	temp = (char *)MALLOC((101)*sizeof(char));
	if (!temp) myerror("error allocating memory");
	fscanf(in,"%s\n",temp);
/*	if (fgets(temp,1000,in)==NULL) myerror("error reading string");*/
	temp=(char *)REALLOC(temp,(1+strlen(temp))*sizeof(char));
	if (!temp) myerror("error reallocating memory");
/*	printf("strlen = %d\n",strlen(temp));*/
	return temp;
}


/************************************************************************/
int **readintegermatrix(FILE *in,int *rows, int *columns)
/*  given a file this reads until it gets a non integer
                                                                        */
{
    int i,j,*data,**toret,count=0,vecsize;
    static char ch;

    vecsize=FIRSTSIZE; /*there is a blank so I can start at 1*/
    data = (int*)MALLOC((vecsize+1)*sizeof(int));
    if (!data) nrerror("error allocating memory in readintegermatrix");

	ch=(char)skipspace(in);
	ungetc(ch,in);
    
    /* first find out how many columns  */
    for (;;) {
		ch = (char)skipblank(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			if (count==vecsize) {
				data = (int*) REALLOC(data,(vecsize+1000+1)*sizeof(int));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
			}
			fscanf(in,"%d",&(data[++count]));
		} else if ((ch =='\n')||(ch == EOF)) {
			*columns = count;
			break;
		}
		else
	    nrerror("This file should just contain integers");
    }
    /* now do the rest of the file  */
    for (;;) {
		ch = (char)skipspace(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			if (count==vecsize) {
				data = (int*) REALLOC(data,(vecsize+1000+1)*sizeof(int));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
			}
			fscanf(in,"%d",&(data[++count]));
		} else {
			ungetc(ch,in);
			break;
		}
    }
    
    if (count%(*columns)!=0) {
        printf("WARNING the file does not *fill* out the matrix count %d\n",
		count);
        *rows = count/(*columns) +1;
    }
    else *rows = count/(*columns);

    toret = nrimatrix(1,*rows,1,*columns);
    count =1;
    for (i=1;i<=*rows;i++) 
	for (j=1;j<=*columns;j++)
	    toret[i][j]=data[count++];
    FREE(data);
    return toret;
}

/************************************************************************/
int **readcharintegermatrix(FILE *in,int *rows, int *columns)
/*  this reads a mixture of integers and single ascii characters and
returns it as an int matrix         */
{
    int i,j,*data,**toret,count=0,vecsize;
    static char ch;

    vecsize=FIRSTSIZE; /*there is a blank so I can start at 1*/
    data = (int*)MALLOC((vecsize+1)*sizeof(int));
    if (!data) nrerror("error allocating memory in readintegermatrix");

	ch=(char)skipspace(in);
	ungetc(ch,in);
    
    /* first find out how many columns  */
    for (;;) {
		ch = (char)skipblank(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			fscanf(in,"%d",&(data[++count]));
			
		} else if (ch=='?') {
			data[++count]=-1;
		} else if (isalpha((int)ch)) {
			data[++count]=(int)ch;
		} else if ((ch =='\n')||(ch == EOF)) {
			*columns = count;
			break;
		} else nrerror("This file should just contain integers or characters");
		if (count==vecsize) {
				data = (int*) REALLOC(data,(vecsize+1000+1)*sizeof(int));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
		}
    }
    /* now do the rest of the file  */
    for (;;) {
		ch = (char)skipspace(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			fscanf(in,"%d",&(data[++count]));
		} else if (ch=='?') {  
			data[++count]=-1;
		} else if (isalpha((int)ch)) {
			data[++count]=(int)ch;
		} else {
			ungetc(ch,in);
			break;
		}
		if (count==vecsize) {
				data = (int*) REALLOC(data,(vecsize+1000+1)*sizeof(int));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
		}
    }
    
    if (count%(*columns)!=0) {
        printf("WARNING the file does not *fill* out the matrix count %d\n",count);
        *rows = count/(*columns) +1;
    }
    else *rows = count/(*columns);

    toret = nrimatrix(1,*rows,1,*columns);
    count =1;
    for (i=1;i<=*rows;i++) 
	for (j=1;j<=*columns;j++)
	    toret[i][j]=data[count++];
    FREE(data);
    return toret;
}

/************************************************************************/
int *readintegervector(FILE *in,int *length)
{
    int *data,count=0,vecsize;
    int ch;

    vecsize=FIRSTSIZE-1; /*there is a blank so I can start at 1*/
	
    data = (int*)MALLOC(FIRSTSIZE*sizeof(int));
    if (!data) nrerror("error allocating memory in readintegermatrix");
	
    for (;;) {
		ch = skipspace(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			if (count==vecsize) {
				data = (int *)REALLOC(data,(vecsize+1000)*sizeof(int));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
			}
			fscanf(in,"%d",&(data[++count]));
		}
		else break;
    }
    data = (int *)REALLOC(data,(count+1)*sizeof(int));
    if (!data) nrerror("error reallocating data");
    *length = count;
    return data;
}
/*********************************************************************/
int findstart(FILE *in,char *pattern)
    /* searched through the file for the first 
     * time that pattern appears, 1 if a success 0 otherwise   */
{
	int c;
   // unsigned long where = 0;
    char *p = pattern,comment='#';
    rewind(in);
	
    while ((c = fgetc(in)) != EOF) {
        if (*p == c)
            p++;
        else {
			if (comment == c) 
				skipline(in);
            p = pattern;
		}
        if (!*p) return 1;
		//   where++;
    }
    return 0;
   /* int c;
    unsigned long where = 0;
    char *p = pattern;
    rewind(in);
  
    while ((c = fgetc(in)) != EOF) {
        if (*p == c)
            p++;
        else
            p = pattern;
        if (!*p)
            return 1;
        where++;
    }
    return 0;*/
}
/*********************************************************************/
int findstartcomment(FILE *in,char *pattern, char comment)
    /* searches through a file for the first 
     * time that pattern appears, 1 if a success 0 otherwise   */
{
    int c;
   // unsigned long where = 0;
    char *p = pattern;
    rewind(in);
	
    while ((c = fgetc(in)) != EOF) {
        if (*p == c)
            p++;
        else {
			if (comment == c) 
				skipline(in);
            p = pattern;
		}
        if (!*p) return 1;
		//   where++;
    }
    return 0;
}
/*********************************************************************/
double nextdouble(FILE *in)
{
    float tmp;
    skipspace(in);
    fscanf(in,"%f",&tmp);
    return (double)tmp;
}

/*********************************************************************/
int nextname(FILE *in, char *filename)
/*  filename must be at least 40 characters long */
{
    int ch,i;
    ch= skipspace(in);
    if (ch==':') ch= skipspace(in);
    for (i=0;i<40;i++) {
		if ((isspace(ch))||ch=='('||ch==EOF) {
			ungetc(ch,in);
			filename[i]='\0';
			return 1;
		}
		filename[i]=(char)ch;
		ch = getc(in);
    }
    return 0;
}
/***************************************************************************/
/*    Now for scanning of parameter files                                  */
/***************************************************************************/
int int_scan(FILE *input, char *namestring,int default_val)
{
    if (findstart(input,namestring)) {
		 return nextint(input);
    } else {
		printf("%s not found in parameter file, using %d\n"
		,namestring,default_val);
			return default_val;
	}
}
/***************************************************************************/
int *intvector_scan(FILE *input, char *namestring,int *default_val)
{
	int *tmp,i;
    if (findstart(input,namestring)) {
		return readintegerline(input);
    } else {
		if (default_val==NULL) {
		//	printf("%s not found in parameter file, using NULL\n",namestring);
			tmp=NULL;
		} else {
			tmp=(int*)MALLOC((default_val[0]+1)*sizeof(int));
			if (!tmp) myerror("error allocating intvector");
			for (i=0;i<=default_val[0];i++) tmp[i]=default_val[i];
		}
	}
	return tmp;	
}
/***************************************************************************/
char char_scan(FILE *input, char *namestring,char default_val)
{
    if (findstart(input,namestring)) {
		 return (char)nextint(input);
    } else {
		printf("%s not found in parameter file, using %c\n"
		,namestring,default_val);
			return default_val;
	}
}
/***************************************************************************/
int *charintvector_scan(FILE *input, char *namestring,int *default_val)
{
	int *tmp;
    if (findstart(input,namestring)) {
		return readcharintegerline(input);
    } else {
		if (default_val==NULL) {
			printf("%s not found in parameter file, using NULL\n",namestring);
			tmp=NULL;
		} else {
			tmp=nrivector(0,1);
			tmp[0]=1;
			tmp[1]=*default_val;
			printf("%s not found in parameter file, using %d\n",namestring,tmp[1]);
		}
	}
	return tmp;	
}
/***************************************************************************/
double double_scan(FILE *input, char *namestring,double default_val)
{
    if (findstart(input,namestring)) {
		 return nextdouble(input);
    } else {
		printf("%s not found in parameter file, using %g\n"
		,namestring,default_val);
		return default_val;
	}
}
/***************************************************************************/
double *doublevector_scan(FILE *input, char *namestring,double *default_val, int *len)
{
	int i;
	double *tmp;
    if (findstart(input,namestring)) {
		 tmp=readdoubleline(input,len);
    } else {
		if (default_val==NULL||(*len)==0) {
			printf("%s not found in parameter file, using NULL\n",namestring);
			tmp=NULL;
			*len=0;
		} else {
			tmp=nrdvector(1,*len);
			for (i=1;i<=*len;i++) tmp[i]=default_val[i];
			printf("%s not found in parameter file, using\n");
			write_dvector(stdout," ",default_val,1,*len);
		}

	}		
	return tmp;
}
/***************************************************************************/
char *namescan(FILE *input, char *namestring, char *default_val)
{
	char *temp=NULL;
	
	if (findstart(input,namestring)) {
		temp = (char *)MALLOC(201*sizeof(char));
		if (!temp) nrerror("error allocating string in namescan");
		if (!nextname(input,temp)) nrerror("error reading datafile name");
    } else if (default_val!=NULL){
		printf("%s not found: using default %s\n",namestring,default_val);
			temp = (char *)MALLOC(201*sizeof(char));
			if (!temp) nrerror("error allocating string in namescan");
			strcpy(temp,default_val);
    } else {
      temp=NULL;
    }
	return temp;
}
/***************************************************************************/
double *sequence_scan(FILE *input, char *namestring,int *n, char *default_val)
{
	double *temp,d[3];
	float fd[3];
	int i;

	if (findstart(input,namestring)) {
		fscanf(input,": %f %f %f",fd,fd+1,fd+2);
		for (i=0;i<3;i++) d[i]=(double)fd[i];
	} else {
		printf("unable to find sequence %s: using default %s\n",namestring,default_val);
		sscanf(default_val,"%f %f %f",fd,fd+1,fd+2);
		for (i=0;i<3;i++) d[i]=(double)fd[i];
    }
	if (fabs(d[1]-d[0])<d[2]) *n=1;
	else *n=(int)((d[1]+d[2]-d[0])/d[2]+0.5);
	temp=nrdvector(1,*n);
	for (i=0;i<*n;i++) temp[i+1]=d[0]+(double)i*d[2];
	return temp;
}
/************************************************************************/
int **readraggedintegerarray(char *filename,int *rows)
/*  given a file this reads to the end of the file
 *  and returns the number of rows - the length oif each row is given by
 *  temp[i][0]                                                 */
{
    FILE *in;
    int i,j,rowstart,rowcount,*data,**toret,count=0,vecsize;
    static char ch;

    vecsize=FIRSTSIZE; /*there is a blank so I can start at 1*/
	in=openinputfile(filename);
	*rows=0;
	
    data = (int*)MALLOC((vecsize+1)*sizeof(int));
    if (!data) nrerror("error allocating memory in readintegermatrix");

	rowstart=0;

	for (;;) {   /*rows*/
		rowcount=0;
		for (;;) {  /* columns  */
			ch = (char)skipblank(in);
			if (isdigit(ch)||ch=='-') {
				ungetc(ch,in);
				if (count==vecsize) {
					data = (int*) REALLOC(data,(vecsize+1000+1)*sizeof(int));
					if (!data) nrerror("error reallocating data");
					vecsize +=1000;
				}
				fscanf(in,"%d",&(data[++count]));
				rowcount++;
			} else break;
		}
		if (rowcount>0) {
			data[rowstart]=rowcount;
			rowstart+=rowcount+1;
			count++;
			*rows+=1;
		}
		
		if (ch==EOF||ch==-1) break;
	}
    /* remake the ragged array  */
	toret=(int **)MALLOC((*rows+1)*sizeof(int *));
	if (!toret) myerror("error allocing memory for ragged integer array");
	count=0;
	for (i=1;i<=*rows;i++) {
		rowcount=data[count++];
		toret[i]=(int*)MALLOC((rowcount+1)*sizeof(int));
			toret[i][0]=rowcount;
			if (!toret[i]) myerror("error allocing memory for ragged integer array");
		for (j=1;j<=rowcount;j++)
			toret[i][j]=data[count++];

	}
	FREE(data);		

	return toret;
}
/************************************************************************/
double **readraggeddoublearray(char *filename,int *rows)
/*  given a file this reads to the end of the file
 *  and returns the number of rows - the length oif each row is given by
 *  temp[i][0]                                                 */
{
    FILE *in;
    int i,j,rowstart,rowcount,count=0,vecsize;
	double *data,**toret;
    static char ch;
	float temf;

    vecsize=FIRSTSIZE; /*there is a blank so I can start at 1*/
	in=openinputfile(filename);
	*rows=0;
	
    data = (double*)MALLOC((vecsize+1)*sizeof(double));
    if (!data) nrerror("error allocating memory in readintegermatrix");

	rowstart=0;

	for (;;) {   /*rows*/
		rowcount=0;
		for (;;) {  /* columns  */
			ch = (char)skipblank(in);
			if (isdigit(ch)||ch=='-') {
				ungetc(ch,in);
				if (count==vecsize) {
					data = (double*) REALLOC(data,(vecsize+1000+1)*sizeof(double));
					if (!data) nrerror("error reallocating data");
					vecsize +=1000;
				}
				fscanf(in,"%g",&temf);
				data[++count]=(double)temf;
				rowcount++;
			} else break;
		}
		if (rowcount>0) {
			data[rowstart]=(double)rowcount;
			rowstart+=rowcount+1;
			count++;
			*rows+=1;
		}
		
		if (ch==EOF||ch==-1) break;
	}
    /* remake the ragged array  */
	toret=(double **)MALLOC((*rows+1)*sizeof(double *));
	if (!toret) myerror("error allocing memory for ragged integer array");
	count=0;
	for (i=1;i<=*rows;i++) {
		rowcount=(int)(data[count++]+0.5);
		toret[i]=(double*)MALLOC((rowcount+1)*sizeof(double));
			toret[i][0]=rowcount;
			if (!toret[i]) myerror("error allocing memory for ragged integer array");
		for (j=1;j<=rowcount;j++)
			toret[i][j]=data[count++];

	}
	FREE(data);		
	return toret;
}
/***************************************************************************/


