#include <math.h>
#include <stdio.h>
#include "myio.h"
#include "nrutil.h"
#include "prior.h"
#include "naiveutil.h"
#include "util.h"
#include "random.h"
#include "patlookup.h"

int **allele_labels,max_alleles;

int *count_alleles(int **d1,int nd1,int **d2,int nd2,int loci)
{
	int *itemp,locus,i,j,count,*alleles;
	
	alleles=nrivector(1,loci);
	max_alleles=0;
	allele_labels=(int **)MALLOC((loci+1)*sizeof(int*));
	if (!allele_labels) myerror("error allocating memory");
	itemp=nrivector(1,2*(nd1+nd2));
	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=nd1;i++) {
			itemp[2*i-1]=d1[i][2*locus-1];
			itemp[2*i]=d1[i][2*locus];
		}
        for (i=1;i<=nd2;i++) {
			itemp[2*nd1+2*i-1]=d2[i][2*locus-1];
			itemp[2*nd1+2*i]=d2[i][2*locus];
		}
		ihpsort(2*(nd1+nd2),itemp);
		if (itemp[1]==-1) alleles[locus]=0;
		else alleles[locus]=1;
		for (i=2;i<=2*(nd1+nd2);i++) {
			if (itemp[i]!=itemp[i-1]) alleles[locus]+=1;
		}
		allele_labels[locus]=nrivector(1,alleles[locus]);
		for (j=1;;j++) if (itemp[j]!=-1) break;
		allele_labels[locus][1]=itemp[j];
		count=1;
		for (i=j+1;i<=2*(nd1+nd2);i++) {
			if (itemp[i]!=itemp[i-1]) 
				allele_labels[locus][++count]=itemp[i];
		}
		for (i=1;i<=alleles[locus];i++) {
			for (j=1;j<=nd1;j++) {
				if (d1[j][2*locus-1]==allele_labels[locus][i])
					d1[j][2*locus-1]=i;
				if (d1[j][2*locus]==allele_labels[locus][i])
					d1[j][2*locus]=i;
			}
			for (j=1;j<=nd2;j++) {
				if (d2[j][2*locus-1]==allele_labels[locus][i])
					d2[j][2*locus-1]=i;
				if (d2[j][2*locus]==allele_labels[locus][i])
					d2[j][2*locus]=i;
			}
		}
		if (alleles[locus]>max_alleles) max_alleles=alleles[locus];
		for (i=1;i<=nd2;i++) {
			for (j=1;j<=alleles[locus];j++) {
				if (d2[i][2*locus-1]==j) break;
				if (d2[i][2*locus-1]==-1) break;
			}
			if (j==alleles[locus]+1) {
				printf("Cannot use multinomial background frequency model\n");
				printf("Allele %d in data is not in background frequency file\n",
					allele_labels[locus][d2[i][2*locus-1]]);
				myerror("1");
			}
			for (j=1;j<=alleles[locus];j++) {
				if (d2[i][2*locus]==j) break;
				if (d2[i][2*locus-1]==-1) break;
			}
			if (j==alleles[locus]+1) {
				printf("Cannot use multinomial background frequency model\n");
				printf("Allele %d in data is not in background frequency file\n",
					allele_labels[locus][d2[i][2*locus]]);
				myerror("2");
			}
		}
	}
	return alleles;
}
/*******************************************************************/
/*   This version of count alleles bins all those that are not in 
sample from the string - ie it just gets those in the matrix d1 
if there are any others in d2 it just puts them all in one bin     */
/*******************************************************************/
int * count_allelesb(int **d1,int nd1,int **d2,int nd2, int loci)
{
	int *itemp,locus,i,j,count,*alleles;
	
	alleles=nrivector(1,loci);
	allele_labels=(int **)MALLOC((loci+1)*sizeof(int*));
	if (!allele_labels) myerror("error allocating memory");
	itemp=nrivector(1,2*nd1);
	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=nd1;i++) {
			itemp[2*i-1]=d1[i][2*locus-1];
			itemp[2*i]=d1[i][2*locus];
		}
		ihpsort(2*nd1,itemp);
		if (itemp[1]==-1) alleles[locus]=0;
		else alleles[locus]=1;
		for (i=2;i<=2*nd1;i++) {
			if (itemp[i]!=itemp[i-1]) alleles[locus]+=1;
		}
		alleles[locus]+=1; /* add one for alleles not in sample */
		allele_labels[locus]=nrivector(1,alleles[locus]);
		for (j=1;;j++) if (itemp[j]!=-1) break;
		allele_labels[locus][1]=itemp[j];
		count=1;
		for (i=j+1;i<=2*nd1;i++) {
			if (itemp[i]!=itemp[i-1]) 
				allele_labels[locus][++count]=itemp[i];
		}
		allele_labels[locus][++count] = 9999;  /* the bin*/
		if (count!=alleles[locus]) myerror("error 1 in count_alleles");
		for (i=1;i<=nd1;i++) {
			if (d1[i][2*locus-1]>0) {
				for (j=1;j<=alleles[locus]-1;j++) {
					if (d1[i][2*locus-1]==allele_labels[locus][j]) {
						d1[i][2*locus-1]=j;
						break;
					}
				}
				if (j==alleles[locus]) {
					printf("j = %d  val = %d\n",j,d1[i][2*locus-1]);
					myerror("error in count_alleles2");
				}
			}
			if (d1[i][2*locus]>0) {
				for (j=1;j<=alleles[locus]-1;j++) {
					if (d1[i][2*locus]==allele_labels[locus][j]) {
						d1[i][2*locus]=j;
						break;
					}
				}
				if (j==alleles[locus]) {
					printf("j = %d  val = %d\n",j,d1[i][2*locus-1]);
					myerror("error in count_alleles3");
				}
			}
		}
		for (i=1;i<=nd2;i++) {
			if (d2[i][2*locus-1]>0) {
				for (j=1;j<=alleles[locus]-1;j++) {
					if (d2[i][2*locus-1]==allele_labels[locus][j]) {
						d2[i][2*locus-1]=j;
						break;
					}
				}
				if (j==alleles[locus]) d2[i][2*locus-1] = alleles[locus];
			} 
			if (d2[i][2*locus]>0) {
				for (j=1;j<=alleles[locus]-1;j++) {
					if (d2[i][2*locus]==allele_labels[locus][j]) {
						d2[i][2*locus]=j;
						break;
					}
				}
				if (j==alleles[locus]) d2[i][2*locus] = alleles[locus];
			}
		}
	}
	return alleles;
}
/*******************************************************************/
void convert(int *x,int loci,int *alleles)
{
	int i,j;
	for (i=1;i<=loci;i++) {
		for (j=1;j<=alleles[i];j++) {
			if (x[2*i-1]==allele_labels[i][j])
				x[2*i-1]=j;
		}
        for (j=1;j<=alleles[i];j++) {
			if (x[2*i]==allele_labels[i][j])
				x[2*i]=j;
		}
	}
}
/*******************************************************************/
void convertb(int *x,int loci,int *alleles)
{
	int i,j;
	for (i=1;i<=loci;i++) {
		for (j=1;j<=alleles[i];j++) {
			if (x[2*i-1]==allele_labels[i][j])
				x[2*i-1]=j;
		}
		if (j==alleles[i]) x[2*i-1]=alleles[i];
        for (j=1;j<=alleles[i];j++) {
			if (x[2*i]==allele_labels[i][j])
				x[2*i]=j;
		}
	    if (j==alleles[i]) x[2*i]=alleles[i];
	}
}
/*******************************************************************/
double **calculate_rho(int **data, int n,int loci, int *alleles, 
					    double before)
{
	int locus,i;
	double **rho;
	max_alleles=imax(alleles,loci);
	rho=nrdmatrix(1,loci,1,max_alleles);
	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=alleles[locus];i++) rho[locus][i]=before;
		for (i=1;i<=n;i++) {
			rho[locus][data[i][2*locus-1]]+=1.0;
			rho[locus][data[i][2*locus]]+=1.0;
		}
	}
	return rho;
}
/*******************************************************************/
double **calculate_pi(int **data, int n, int loci, int *alleles)
{
	int locus,i;
	double **rho;
	max_alleles=imax(alleles,loci);
	rho=nrdmatrix(1,loci,1,max_alleles);
	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=alleles[locus];i++) rho[locus][i]=0.0;
		for (i=1;i<=n;i++) {
			rho[locus][data[i][2*locus-1]]+=1.0;
			rho[locus][data[i][2*locus]]+=1.0;
		}
		for (i=1;i<=alleles[locus];i++) rho[locus][i]/=2.*n;
	}
	return rho;
}
/*******************************************************************/
extern long replicates;
/*******************************************************************/
int *getalleles(int **data1,int **data2, int n1,int n2, int loci)
{
	int locus,i,*a,mx;
	
	a=nrivector(1,loci);
	for (locus=1;locus<=loci;locus++) {
		mx=0;
		for (i=1;i<=n1;i++) {
			if (data1[i][2*locus-1]>mx)
				mx=data1[i][2*locus-1];
			if (data1[i][2*locus]>mx)
				mx=data1[i][2*locus];
		}
		for (i=1;i<=n2;i++) {
			if (data2[i][2*locus-1]>mx)
				mx=data2[i][2*locus-1];
			if (data2[i][2*locus]>mx)
				mx=data2[i][2*locus];
		}
		a[locus]=mx;
	}
	return a;
}
/*******************************************************************/
int loc;
int compare(const void *g1,const void *g2)
{
	int l,*gen1,*gen2;
	gen1=*(int**)g1;
	gen2=*(int**)g2;
	for (l=1;l<=loc;l++) {
		if (gen1[2*l-1]<gen2[2*l-1]) return -1;
		else if (gen1[2*l-1]>gen2[2*l-1]) return 1;
		if (gen1[2*l]<gen2[2*l]) return -1;
		else if (gen1[2*l]>gen2[2*l]) return 1;
	}
	return 0;
}
/*******************************************************************/
void sort_data(int **x, int n, int loci)
{
	int i,l,itmp;

	for (i=1;i<=n;i++) {
		for (l=1;l<=loci;l++) {
			if (x[i][2*l-1]>x[i][2*l]) {
				itmp=x[i][2*l];
				x[i][2*l]=x[i][2*l-1];
				x[i][2*l-1]=itmp;
			}
		}
	}
	loc=loci;
	qsort((void *)(x+1),(size_t)n,sizeof(int*),compare);
}
/*******************************************************************/
model read_model(char *filename)
{
	char *datafilename,*freqfile;
	int **background_freq,floci,fsamples,i,j;
	model m;
	FILE *in;

	prior nllprior={NULLPRIOR,{0,0}};

	in=openinputfile(filename);

	m.fatherprior=prior_scan(in,"fatherprior",nllprior);
	m.motherprior=prior_scan(in,"motherprior",nllprior);
	m.alphaprior=prior_scan(in,"alphaprior",nllprior);
	m.betaprior=prior_scan(in,"betaprior",nllprior);

	if (m.fatherprior.prtype==NULLPRIOR&&
		m.alphaprior.prtype==NULLPRIOR)
		myerror("error at least one of alphaprior and fatherprior must be defined");

	if (m.alphaprior.prtype==NULLPRIOR) m.foffmodel=1;
	else if (m.fatherprior.prtype==NULLPRIOR) m.foffmodel=3;
	else m.foffmodel=2;
	
	if (m.motherprior.prtype==NULLPRIOR&&
		m.betaprior.prtype==NULLPRIOR)
		myerror("error at least one of betaprior and motherprior must be defined");
	if (m.betaprior.prtype==NULLPRIOR) m.moffmodel=1;
	else if (m.motherprior.prtype==NULLPRIOR) m.moffmodel=3;
	else m.moffmodel=2;

	m.freqmodel=int_scan(in,"freqmodel",1);
	replicates=int_scan(in,"replicates",1000);
  
	datafilename=namescan(in,"datafile","datafile");
	background_freq
		=readintegermatrix(openinputfile(datafilename),&m.n,&m.loci);
	m.loci /=2;
	m.data=(int**)malloc((m.n+1)*sizeof(int*));
	if (!m.data) myerror("error allocating m.data");
	for (i=1;i<=m.n;i++) {
		m.data[i]=(int*) malloc((2*m.loci+1)*sizeof(int));
		if (!m.data[i]) myerror("error allocating m.data[i]");
		for (j=1;j<=2*m.loci;j++) 
			m.data[i][j]=background_freq[i][j];
	}
	free_imatrix(background_freq,1,1);
	sort_data(m.data,m.n,m.loci);

	freqfile=namescan(in,"freqfile",NULL);
	if (freqfile==NULL) {
		printf("no frequency file given, have to use observed alleles\n");
		fsamples=0;
		FREE(freqfile);
	} else {
		background_freq
			=readintegermatrix(openinputfile(freqfile),&fsamples,&floci);
		if (floci/2!=m.loci) 
			myerror("incorrect loci in frequency file");
	}
	m.alleles=getalleles(background_freq,m.data,fsamples,m.n,m.loci);
	if (m.freqmodel==1) {
		m.rho=calculate_pi(background_freq,fsamples,m.loci,m.alleles);
	} else {
		m.rho=calculate_rho(background_freq,fsamples,m.loci,m.alleles,1.0);
	}
	write_raggeddoublearray(stdout,m.rho,m.loci,m.alleles);

	FREE(datafilename);
	fclose(in);
	return m;
}
/*******************************************************************/
crypt_model read_crypt_model(char *filename)
{
	char *datafilename,*freqfile;
	int **background_freq,floci,fsamples,i,j;
	crypt_model m;
	FILE *in;

	in=openinputfile(filename);

	m.freqmodel=int_scan(in,"freqmodel",1);
	replicates=int_scan(in,"replicates",1000);
  
	datafilename=namescan(in,"datafile","datafile");
	background_freq
		=readintegermatrix(openinputfile(datafilename),&m.n,&m.loci);
	m.loci /=2;
	m.data=(int**)malloc((m.n+1)*sizeof(int*));
	if (!m.data) myerror("error allocating m.data");
	for (i=1;i<=m.n;i++) {
		m.data[i]=(int*) malloc((2*m.loci+1)*sizeof(int));
		if (!m.data[i]) myerror("error allocating m.data[i]");
		for (j=1;j<=2*m.loci;j++) 
			m.data[i][j]=background_freq[i][j];
	}
	free_imatrix(background_freq,1,1);

	freqfile=namescan(in,"freqfile",NULL);
	if (freqfile==NULL) {
		printf("no frequency file given, have to use observed alleles\n");
		fsamples=0;
		FREE(freqfile);
	} else {
		background_freq
			=readintegermatrix(openinputfile(freqfile),&fsamples,&floci);
		if (floci/2!=m.loci) 
			myerror("incorrect loci in frequency file");
	}
	m.mother=intvector_scan(in,"mother",NULL);
	if (m.freqmodel) {
		m.alleles=count_alleles(m.data,m.n,background_freq,fsamples,m.loci);
		convert(m.mother,m.loci,m.alleles);
		m.rho=calculate_pi(background_freq,fsamples,m.loci,m.alleles);
	} else {
		m.alleles=count_allelesb(m.data,m.n,background_freq,fsamples,m.loci);
		convertb(m.mother,m.loci,m.alleles);
		m.rho=calculate_rho(background_freq,fsamples,m.loci,m.alleles,1.0);
	}
	write_raggeddoublearray(stdout,m.rho,m.loci,m.alleles);

	FREE(datafilename);
	fclose(in);
	return m;
}
/********************************************************************************/
int *generate_genotype(int loci, int *alleles, double **rho, int freqmodel)
{
	int *x,locus;

	x = (int*)malloc((2*loci+1)*sizeof(int));
	if (!x) myerror("error allocating x");
	if (freqmodel==1) {
		for (locus=1;locus<=loci;locus++) {
			multiple_gen_from_absprobs(x+2*locus-2,2,rho[locus],alleles[locus]);
		}
	} else {
		for (locus=1;locus<=loci;locus++) {
			rmultidirichletc(x+2*locus-2,rho[locus],alleles[locus],2);
		}
	}
	return x;
}
/********************************************************************************/
parents generate_parents(prior parprior,prior aprior, int loci, int *alleles,int n, 
						 double **rho, int freqmodel, int offmodel)
{
	double x,alpha;	
	int i;
	parents p;

	p.parentage=nrivector(1,n);
	if (offmodel==3) {
		sample_prior(&alpha,aprior);
		p.n=rEwens(p.parentage,alpha,n);
	} else {
		for (;;) {
			sample_prior(&x,parprior);
			p.n=(int)(x+0.5);
			if (p.n>0) break;
		}
		if (p.n==1) {
			for (i=1;i<=n;i++) p.parentage[i]=1;
		} else if (offmodel==1) {
			for (i=1;i<=n;i++) p.parentage[i]=runiformint(1,p.n);
		} else {
			sample_prior(&alpha,aprior);
			rexchangeablemultidirichletc(p.parentage,alpha,p.n,n);
		}
	}
	
	p.genotype=(int**)malloc((p.n+1)*sizeof(int *));
	if (!p.genotype) myerror("error allocating genotype");
	for (i=1;i<=p.n;i++)
		p.genotype[i]=generate_genotype(loci,alleles,rho,freqmodel);

	return p;
}
/********************************************************************************/
int  *mating(int *g1, int *g2, int loci)
{
	int i,*d;
	double r;

	d=(int*)malloc((2*loci+1)*sizeof(int));
	if (!d) myerror("error allocating d"); 
	
	for (i=1;i<=loci;i++) {
		r=ranDum();
		if (r<0.25) {
			d[2*i-1]=g1[2*i-1];
			d[2*i]=g2[2*i-1];
		} else if (r<0.5) {
			d[2*i-1]=g1[2*i-1];
			d[2*i]=g2[2*i];
		} else if (r<0.75) {
			d[2*i-1]=g1[2*i];
			d[2*i]=g2[2*i-1];
		} else {
			d[2*i-1]=g1[2*i];
			d[2*i]=g2[2*i];
		}
	}
	return d;
}
/********************************************************************************/
int **mate(parents *fathers, parents *mothers, int loci, int n)
{
	int i,**x;
	
	x=(int**)malloc((n+1)*sizeof(int*));
	if (!x) myerror("error allocating x");
	for (i=1;i<=n;i++) {
		x[i]=mating(fathers->genotype[fathers->parentage[i]],
			mothers->genotype[mothers->parentage[i]],loci);
	}
	return x;
}
/********************************************************************************/
sample generate_sample(model *m)
{
	sample s;

	s.fathers=generate_parents(m->fatherprior,m->alphaprior,m->loci,m->alleles,m->n,m->rho,
		m->freqmodel,m->foffmodel);
	s.mothers=generate_parents(m->motherprior,m->betaprior,m->loci,m->alleles,m->n,m->rho,
		m->freqmodel,m->moffmodel);
	s.x=mate(&s.fathers,&s.mothers,m->loci,m->n);
	sort_data(s.x,m->n,m->loci);

	return s;
}
/********************************************************************************/
void destroy_parents(parents *p)
{
	int i;
	free_ivector(p->parentage,1);
	for (i=1;i<=p->n;i++) FREE(p->genotype[i]);
	FREE(p->genotype);
	return;
}
/********************************************************************************/
void destroy_sample( sample *s,int n)
{
	int i;
	for (i=1;i<=n;i++) FREE(s->x[i]);
	FREE(s->x);
	destroy_parents(&s->fathers);
	destroy_parents(&s->mothers);
	return;
}
/********************************************************************************/
int matches(model *m, sample *s)
{
	int i,locus;

	for (i=1;i<=m->n;i++) {
		for (locus=1;locus<=m->loci;locus++) {
			if (m->data[i][2*locus-1]!=s->x[i][2*locus-1]||
				m->data[i][2*locus]!=s->x[i][2*locus])
				return 0;
		}
	}
	return 1;
}
/********************************************************************************/
int possible_father(int *father, int *mother, crypt_model *m)
{
	int i,j;

	for (i=1;i<=m->n;i++) {
		for (j=1;j<=m->loci;j++) {
			if (lookup_muts(m->data[i]+2*j-2,mother+2*j-2,father+2*j-2)>0) break;
		}
		if (j==m->loci+1) return 1; 
	}
	return 0;
}
/********************************************************************************/