/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include "offmodel.h"
#include "prior.h"
#include "nrutil.h"
#include "util.h"
#include "myio.h"
#include <math.h>
/***********************************************************/
extern int samples;
/***********************************************************/
double ll_off(int *off, int k, double a, offmodel *o)
{
	double lp;
	int i;
#ifdef CHECK
	check_offb(off,k,o->model);
#endif
	switch (o->model) {
	case 1:
		return -samples*(log((double)k)) + lfactrl(k);
	case 2: 
		lp = log_dmulti_dirichletb(off,a,k) + lfactrl(k) - lfactrl(samples);
		for (i=1;i<=k;i++) lp += lfactrl(off[i]);
		return lp;
	case 3:
		lp = (double)k*log(a) + lgamma(a) 
			-lgamma((double)samples+a);
		for (i=1;i<=k;i++) lp +=lgamma((double)off[i]);
		return lp;
	default:
		myerror("problem in ll_off");
		return 0.0;
	}
}
/***********************************************************/
double loff_prior(int k, double a, offmodel *o)
{
	double x;
	switch(o->model) {
	case 1:
		x=(double)k;
		return log_prior(&x,o->nprior);
	case 2:
		x=(double)k;
		return log_prior(&x,o->nprior) + log_prior(&a,o->alphaprior);
	case 3:
		return log_prior(&a,o->alphaprior);
	default:
		myerror("whould not get here");
		return 0.0;
	}
}
/***********************************************************/
double lpost_off(int *off, int k, double a, offmodel *o, double temper)
{	
	return loff_prior(k,a,o)/temper + ll_off(off,k,a,o);
}
/***********************************************************/
void copy_offmodel(offmodel *to, offmodel from)
{
	to->model=from.model;
	copyprior(&to->alphaprior,from.alphaprior);
	copyprior(&to->nprior,from.nprior);
}
/***********************************************************/
double *lproboff(int *o1,int k1,double alpha, offmodel *off1,
				int *o2, int k2,double beta, offmodel *off2, int n)
{
	
	double *temp;
	int i,j;
	temp=nrdvector(1,k1*k2);
	if (off1->model==1&&off2->model==1) {
		for (i=1;i<=k1*k2;i++) temp[i]=0.0;
	} else if (off1->model==2&&off2->model==2) {
		for (i=0;i<k1;i++) {
			for (j=0;j<k2;j++) {
				temp[i*k2+j]=LOG(alpha+(double)o1[i+1])
					+LOG(beta+(double)o2[j+1])
					-LOG(beta+(double)(n-1)) -LOG(alpha+(double)(n-1));
			}
		}
	} else if (off1->model==3&&off2->model==3) {
		for (i=0;i<k1;i++) {
			for (j=0;j<k2;j++) {
				if (o1[i+1]==0) temp[i*k1+j]=LOG(alpha)-LOG(alpha+(double)(n-1));
				else temp[i*k2+j]=LOG((double)o1[i+1])-LOG(alpha+(double)(n-1));
				
				if (o2[j+1]==0) 
					temp[i*k2+j]+=LOG(beta)-LOG(beta+(double)(n-1));
				else temp[i*k2+j]+=LOG((double)o2[j+1])-LOG(beta+(double)(n-1));
			}
		}
	} else myerror("using mixed models - not supported yet");
	
	return temp;
}
/***********************************************************/
int check_off(int *p, int *o, int k, int model)
{
	int i;
	int *off;
	
	off=nrivector0(1,k);
	for (i=1;i<=samples;i++) {
		if (p[i]<1||p[i]>k) 
			myerror("error a");
		off[p[i]]+=1;
	}
	for (i=1;i<=k;i++) {
		if (off[i]!=o[i]) {
			printf("off %d calc %d i %d k %d\n",o[i],off[i],i,k);
			write_ivector(stdout," ",o,1,k);
			write_ivector(stdout," ",p,1,samples);
			myerror("error 1");
		}
		if (model==3) {
			if (o[i]==0) myerror("error 2");
		}
	}
	free_ivector(off,1);
	return 1;	
}
/***********************************************************/
int check_offb(int *o, int k, int model)
{
	int i,tmp1=0,tmp2=0;
	
	for (i=1;i<=k;i++) {
		tmp1+=(o[i]>0);
		tmp2+=o[i];
	}
	if (model==3) { 
		if (tmp1!=k) myerror(" ec");
	}
	if (tmp2!=samples) 
		myerror("ed");
	return 1;	
}
/***********************************************************/

