/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <stdio.h>
#include "myio.h"
#include "nrutil.h"
#include "patlookup.h"
#include "output.h"
/******************************************************/
output create_output(int n, int loci)
{
	output tmp;
	tmp.n=n;
	tmp.loci=loci;
	tmp.mutations=nrimatrix0(1,n,1,loci);
	tmp.shared_fathersmothers=nrimatrix0(1,n,1,n);
	tmp.shared_parents=nrimatrix0(1,n,1,n);
	return tmp;
}
/******************************************************/
int destroy_output(output *any)
{
	free_imatrix(any->shared_parents,1,1);
	free_imatrix(any->shared_fathersmothers,1,1);
	free_imatrix(any->mutations,1,1);
	any->n=0;
	any->loci=0;
	return 1;
}
/********************************************************************/
/* writes the set of paternity maternity and mutation files         */
/********************************************************************/
int write_output(char *filestem, output *any)
{
	FILE *out;

    write_imatrix(out=openstemleafoutputfile(filestem,"paternity"),
	any->shared_fathersmothers,1,any->n,1,any->n);
	fclose(out);

	write_imatrix(out=openstemleafoutputfile(filestem,"parentage"),
		any->shared_parents,1,any->n,1,any->n);
	fclose(out);

	write_imatrix(out=openstemleafoutputfile(filestem,"muts"),
		any->mutations,1,any->n,1,any->loci);
	fclose(out);
	
	return 1;
}
/********************************************************************/
/*  updates the shared maternity, paternity, parentage and mutations*/
/********************************************************************/
void update_output(output *o,int *pf, int *pm, int **Y, 
				   int **gf, int **gm) 
{
	int j,k;
	
	for (j=1;j<=o->n;j++) {
		for (k=1;k<j;k++) {
			if (pf[j]==pf[k]) {
				o->shared_fathersmothers[k][j]+=1;
				if (pm[j]==pm[k]) {
					o->shared_fathersmothers[j][k]+=1;
					o->shared_parents[j][k]+=1;
					o->shared_parents[k][j]+=1;
				} 
			} else if (pm[j]==pm[k]) {
				o->shared_fathersmothers[j][k]+=1;
				o->shared_parents[j][k]+=1;
			}
		}
		o->shared_fathersmothers[j][j]+=1;
		o->shared_parents[j][j]+=1;
		for (k=1;k<=o->loci;k++) {
			o->mutations[j][k]+=lookup_muts(Y[j]+2*k-2,
				gf[pf[j]]+2*k-2,gm[pm[j]]+2*k-2);
		}
	}
}
/********************************************************************/
