/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <stdio.h>
#include <math.h>
#include "prior.h"
#include "util.h"
#include "parameters.h"
#include "myio.h"

/**********************************************************************/
void printparameters(FILE *out,parameters *p)
{
	fprintf(out,"burn-in: %d\n",p->warmup);
	fprintf(out,"samples: %d\n",p->replicates);
	fprintf(out,"seed: %d\n",p->seed);
	fprintf(out,"thinning: %d\n",p->between);
	fprintf(out,"independence: %d\n",p->independence);
	fprintf(out,"format: %c\n",p->format);
	fprintf(out,"datafile: %s\n",p->datafile);
	fprintf(out,"freqmodel: %d\n",p->freqmodel);
	if (p->freqfile) fprintf(out,"freqfile: %s\n",p->freqfile);
	if (p->fatherfile) {
		fprintf(out,"fatherfile: %s\n",p->fatherfile);
		if (p->knownfathers) fprintf(out,"knownfathers: %s\n",p->knownfathers);
	}
	if (p->motherfile) {
		fprintf(out,"motherfile: %s\n",p->motherfile);
		if (p->knownmothers) fprintf(out,"knownmothers: %s\n",p->knownmothers);
	}
		
	if (p->initfile) fprintf(out,"p->initialfile: %s\n",p->initfile);
	fprintf(out,"fatherrange: %d %d\n",p->fatherrange[1],p->fatherrange[2]);
	fprintf(out,"motherrange: %d %d\n",p->motherrange[1],p->motherrange[2]);

	fprintf(out,"fatherprior: ");
	printprior(out,p->fatherprior,"\n");
	fprintf(out,"motherprior: ");
	printprior(out,p->motherprior,"\n");
	fprintf(out,"muprior: ");
	printprior(out,p->muprior,"\n");
	fprintf(out,"chains: ");
	if (p->tempering) write_dvector(out," ",p->tempvals,1,p->tempering);
	else fprintf(out,"0");
	if (p->alphaprior.prtype!=NULLPRIOR) {
		fprintf(out,"alphaprior: ");
		printprior(out,p->alphaprior,"\n");
	}
	if (p->alphaprior.prtype!=NULLPRIOR) {
		fprintf(out,"betaprior: ");
		printprior(out,p->alphaprior,"\n");
	}
	fprintf(out,"\n------------------------------------------------\n");
}
/*****************************************************************************/
parameters readparameters(char *filename)
{
	FILE *in;
	parameters p;
	int rang[3]={2,1,10};
	prior nllprior={NULLPRIOR};
	prior mutprior={GAMMA,{1,1000}};  // an uninformative prior

	in=openinputfile(filename);
	p.warmup=int_scan(in,"burn-in",100);
	p.between=int_scan(in,"thinning",100);
	p.replicates=int_scan(in,"samples",1000);
	p.freqmodel=int_scan(in,"freqmodel",0);
	p.useloci=intvector_scan(in,"useloci",NULL);
	p.usesamp=intvector_scan(in,"usesamp",NULL);
	p.fatherrange=intvector_scan(in,"fatherrange",rang);
	p.motherrange=intvector_scan(in,"motherrange",rang);
	p.seed=int_scan(in,"seed",1);

	p.datafile=namescan(in,"datafile","datafile");
	
	p.tempvals=doublevector_scan(in,"chains",NULL,&p.tempering);
	if (p.tempering==1&&fabs(p.tempvals[1])<0.001) p.tempering=0;
	
    p.freqfile=namescan(in,"freqfile",NULL);

	p.fatherfile=namescan(in,"fatherfile",NULL);
	p.motherfile=namescan(in,"motherfile",NULL);
	p.initfile = namescan(in,"initialfile",NULL);
	p.independence=int_scan(in,"independence",0);
	p.format=char_scan(in,"format",'N');
		
	p.fatherprior=prior_scan(in,"fatherprior",nllprior);
	p.alphaprior=prior_scan(in,"alphaprior",nllprior);
	p.motherprior=prior_scan(in,"motherprior",nllprior);
	p.betaprior=prior_scan(in,"betaprior",nllprior);	
	p.muprior=prior_scan(in,"muprior",mutprior);
	p.knownmothers=namescan(in,"knownmothers",NULL);
	p.knownfathers=namescan(in,"knownfathers",NULL);
	
	fclose(in);
	return p;
}
/*****************************************************************************/
void destroy_parameters(parameters *p)
{
	if (p->datafile!=NULL) FREE(p->datafile);
	if (p->freqfile!=NULL) FREE(p->freqfile);
	if (p->fatherfile!=NULL) FREE(p->fatherfile);
	if (p->motherfile!=NULL) FREE(p->motherfile);
	if (p->knownmothers!=NULL) FREE(p->knownmothers);
	if (p->knownfathers!=NULL) FREE(p->knownfathers);
	if (p->initfile!=NULL) FREE(p->initfile);
	p->warmup=p->seed=p->between=p->tempering=p->freqmodel=p->independence=0;
	if (p->tempvals!=NULL) FREE(p->tempvals);
	FREE(p->fatherrange);
	FREE(p->motherrange);
	if (p->useloci!=NULL) FREE(p->useloci);
	if (p->usesamp!=NULL) FREE(p->usesamp);
}
/*****************************************************************************/
