/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <math.h>
#include "nrutil.h"
#include "util.h"
#include "random.h"
#include "patlookup.h"
#include "prior.h"
#include "myio.h"
#include "parents.h"
#include "data.h"
#include "patutil.h"
/*******************************************************************/
extern int samples, loci;
/*******************************************************************/
double nparentsprior(parents *p)
{
	double x;
	if (p->off.model==1) {
		x=(double)p->n;
		return log_prior(&x,p->off.nprior);
	} else return 0.0;
}
/*******************************************************************/
parents copy_parents(parents *p)
{
	parents tmp;

	tmp.n=p->n;
	tmp.range[0]=p->range[0];
	tmp.range[1]=p->range[1];
	tmp.splitparentage=p->splitparentage;
	tmp.joinparentage=p->joinparentage;
	tmp.gibbsparentaloffspring=p->gibbsparentaloffspring;
	tmp.nmissing=p->nmissing;
	if (tmp.nmissing) {
		tmp.mssl=copy_ivector(p->mssl,1,tmp.nmissing);
        tmp.mssp=copy_ivector(p->mssp,1,tmp.nmissing);
	}
	copy_offmodel(&tmp.off,p->off);
	tmp.alpha=p->alpha;
	tmp.parentage=copy_ivector(p->parentage,1,samples);
	if (p->known!=NULL)
		tmp.known=copy_ivector(p->known,0,samples);	
	else tmp.known=NULL;
	tmp.offspring=copy_ivector(p->offspring,1,p->range[1]);
	tmp.fixed=p->fixed;
	tmp.genotype=copy_imatrix(p->genotype,1,p->range[1],1,2*loci);
	return tmp;
}
/*******************************************************************/
void destroy_parents(parents *p)
{
	free_imatrix(p->genotype,1,1);
	free_ivector(p->parentage,1);
	free_ivector(p->offspring,1);
	if (p->known) free_ivector(p->known,0);
	if (p->nmissing) {
		free_ivector(p->mssl,1);
		free_ivector(p->mssp,1);
	}
	p->fixed=0;
	p->range[1]=0;p->n=0;
}
/***********************************************************/
int check_parents(parents *p)
{
    int i,*off;
	
	off=nrivector0(1,p->n);
	for (i=1;i<=samples;i++) {
		if (p->parentage[i]<1||p->parentage[i]>p->n) 
			myerror("error a");
		off[p->parentage[i]]+=1;
	}
	for (i=1;i<=p->n;i++) {
		if (p->offspring[i]!=off[i]) 
			myerror("error 1");
		if (p->offspring[i]==0) 
			myerror("error 2");
	}
	free_ivector(off,1);
	return 1;		
}
/*******************************************************************/
void copy_parentsinplace(parents *to,parents *from)
// assume everything else remains the same
{
	int i,j;
	to->n=from->n;
	for (i=1;i<=samples;i++) to->parentage[i]=from->parentage[i];
	for (i=1;i<=to->n;i++)
		for (j=1;j<=2*loci;j++) 
			to->genotype[i][j]=from->genotype[i][j];
}
/*******************************************************************/
/*  takes a pointer to a list of parents and two integers which give
    two positions.  Swaps the labelling of the parents at positions
	a and b                                                        */
/*******************************************************************/
void swap_parents(parents *p,int a, int b)
{
	int i,tmp;
	
	for (i=1;i<=2*loci;i++) {
		tmp=p->genotype[a][i];
		p->genotype[a][i]=p->genotype[b][i];
		p->genotype[b][i]=tmp;
	}
	for (i=1;i<=samples;i++) {
		if (p->parentage[i]==a) {
			p->parentage[i]=b;
		} else if (p->parentage[i]==b) {
			p->parentage[i]=a;
		}
	}
	tmp=p->offspring[a];
	p->offspring[a]=p->offspring[b];
	p->offspring[b]=tmp;
}
/*******************************************************************/
void write_parents(FILE *out, char *mess, parents *p)
{
    fprintf(out,"%s: %d %d %d %d\n",mess,p->n,p->fixed,p->range[0],p->range[1]);
    fprintf(out,"nprior: ");
	printprior(out,p->off.nprior,"\n");
	fprintf(out,"alphaprior: ");
	printprior(out,p->off.alphaprior,"\n");
	write_imatrix(out,p->genotype,1,p->n,1,2*loci);
	fprintf(out,"parentage\n");
	write_ivector(out," ",p->parentage,1,samples);
	if (p->fixed&&p->known) 
	write_ivector(out," ",p->known,1,samples);
	return;
}
/********************************************************************/
parents read_parents(FILE *in,char *namestring)
{
	int *a,i,j,**imat;
	char trash[40];
	parents p;
	extern prior nllprior;
	
	if (findstart(in,namestring)) { 
		a=readintegerline(in);
    } else {
		printf("%s not found in parameter file\n");
	    myerror("cannot continue");
		p.n=0;return p; //just to stop warning messages
	}
	p.n=a[1];
	p.fixed=a[2];
	p.range[0]=a[3];
	p.range[1]=a[4];
	FREE(a);
	p.off.nprior=prior_scan(in,"nprior",nllprior);
	p.off.alphaprior=prior_scan(in,"alphaprior",nllprior);
	imat=readintegermatrix(in,&i,&j);
	p.genotype=nrimatrix(1,p.range[1],1,2*loci);
	for (i=1;i<=p.n;i++) 
		for (j=1;j<=2*loci;j++)
			p.genotype[i][j]=imat[i][j];
	FREE(imat);
	fscanf(in,"%s\n",trash);
	p.parentage=readintegervector(in,&i);
	if (p.fixed) p.known=readintegervector(in,&i);
	else p.known=NULL;
	if (i!=samples) myerror("error with number of parents");

	return p;
}
/*******************************************************************/
parents assign_parents(prior pv,prior pa,int **pg, int np,int *range, int **Y)
{
	int i,j,wh;
	parents p;

	copyprior(&p.off.nprior,pv);
	copyprior(&p.off.alphaprior,pa);
	if (pa.prtype!=NULLPRIOR) {
		if (pv.prtype==NULLPRIOR) {
			p.splitparentage=splitparentageb;
			p.joinparentage=joinparentageb;
			p.gibbsparentaloffspring=gibbsparentaloffspringb;
			p.off.model=3;
		} else {
			p.gibbsparentaloffspring=gibbsparentaloffspringa;
			p.splitparentage=splitparentagea;
			p.joinparentage=joinparentagea;
			p.off.model=2;
		} 
	} else {
		p.splitparentage=splitparentagea;
		p.joinparentage=joinparentagea;
		p.gibbsparentaloffspring=gibbsparentaloffspringa;
		p.off.model=1;
		if (pv.prtype==NULLPRIOR) {
			p.off.nprior.prtype=UNIFORM;
			p.off.nprior.par[0]=range[1];
			p.off.nprior.par[1]=range[2];
			p.n=runiformint(p.range[0],p.range[1]);
		}
	}
	if (pv.prtype==UNIFORM&&pv.par[0]<pv.par[1]) {
		if (pv.par[0]<1.0) {
			mywarning("uniform prior for parent number "
				"\n lower bound must be at least 1\n");
			pv.par[0]=1.0;
		}
		p.range[0]=(int)(pv.par[0]+0.5);
		p.range[1]=(int)(pv.par[1]+0.5);
	} else {
		p.range[0]=range[1];
		p.range[1]=range[2];
	}
	if (samples<p.range[1]&&p.off.model==3) p.range[1]=samples;

	printf("Using model %d\n",p.off.model);
	
	if (pv.prtype==CONSTPRIOR) {
		p.range[0]=p.range[1]=(int)(pv.par[0]+0.5);
	} 
	p.n=p.range[1];
	if (p.off.model==1) p.alpha=0.0;
	else {
		sample_prior(&p.alpha,p.off.alphaprior);
		if (p.alpha<0.01||p.alpha>10000) p.alpha=1.0;  //avoid ridiculous starting values
	}
	
	if (pg==NULL) 	{
		p.genotype=nrimatrix(1,p.range[1],1,2*loci);
		for (i=1;i<=p.n;i++) {
			wh = runiformint(1,samples);
			for (j=1;j<=2*loci;j++) {
				if (Y[wh][j]>0)
					p.genotype[i][j]=Y[wh][j];
				else p.genotype[i][j]=1;
			}
		}
		p.fixed=0;
		p.nmissing=0;
	} else {		
		p.n=np;		
		p.fixed=np;
		if (p.fixed>p.range[0]) p.range[0]=p.fixed;
		p.genotype=nrimatrix(1,p.range[1],1,2*loci);
		p.nmissing=0;
		for (i=1;i<=p.n;i++) {
			for (j=1;j<=loci;j++) {
				if (pg[i][2*j-1]<0) p.nmissing+=1;
				p.genotype[i][2*j-1]=pg[i][2*j-1];
                p.genotype[i][2*j]=pg[i][2*j];
			}
		}
		if (p.nmissing) {
			p.mssl=nrivector(1,p.nmissing);
			p.mssp=nrivector(1,p.nmissing);
			for (wh=0,i=1;i<=p.n;i++) {
				for (j=1;j<=loci;j++) {
					if (pg[i][2*j-1]<0) {
						p.mssl[++wh]=j;
						p.mssp[wh]=i;
					}
				}
			}
		}
	}
	p.parentage=nrivector(1,samples);
	p.offspring =nrivector(1,p.range[1]);
	p.known=NULL;
	
	return p;
}
/*******************************************************************/
void assign_offspring_random(parents *p1,parents *p2)
{
	int i;
	
	if (p1->n==1) {
		for (i=1;i<=samples;i++) p1->parentage[i]=1;
	} else {
		if (p1->known) {
			for (i=1;i<=samples;i++) {
				if (!p1->known[i]) {
					p1->parentage[i]=runiformint(1,p1->n);
				} else p1->parentage[i]=p1->known[i];
			}
		} else {
			if (p1->n>samples) {
				for (i=1;i<=samples;i++) 	p1->parentage[i]=i;	
			} else {
				for (i=1;i<=p1->n;i++) 	p1->parentage[i]=i;	
				for (i=p1->n+1;i<=samples;i++) p1->parentage[i]=runiformint(1,p1->n);
			}	
		}
	}
	if (p2->n==1) {
		for (i=1;i<=samples;i++) p2->parentage[i]=1;
	} else {
		if (p2->known) {
			for (i=1;i<=samples;i++) {
				if (!p2->known[i]) {
					p2->parentage[i]=runiformint(1,p2->n);
				} else p2->parentage[i]=p2->known[i];
			}
		} else {
			for (i=1;i<=p2->n;i++) p2->parentage[i]=i;
			for (i=p2->n+1;i<=samples;i++) p2->parentage[i]=runiformint(1,p2->n);
		}
	}
	for (i=1;i<=p1->n;i++) p1->offspring[i]=0;
	for (i=1;i<=p2->n;i++) p2->offspring[i]=0;
	for (i=1;i<=samples;i++) {
		p1->offspring[p1->parentage[i]]+=1;
		p2->offspring[p2->parentage[i]]+=1;
	}
	return;
}
/*************************************************************/
void resample_parents(parents *p,parents *ch)//, double *proploc)
{
    int i;
	sample_n(p);
	if (p->fixed>p->n) p->n=p->fixed;
	if (p->fixed==p->n) {
		sampleparentaloffspring(p,ch,1.0,1);//,proploc);
	} else {
		for (i=1;i<=p->n;i++) {
			p->offspring[i]=1;
			p->parentage[i]=i;
		}
		for (i=p->n+1;i<=samples;i++) {
			p->parentage[i]=runiformint(1,p->n);
			p->offspring[p->parentage[i]]+=1;
		}
		gibbsparentalgenotypes(p,ch,1.0,1.0,1);  //,proploc);
	}
}
/*************************************************************/
/* four utility functions for splitjoin                      */
/*************************************************************/
double splitparentagea(parents *p, double *lpback)
{
	int i,wh,*change,n1,count;
	double lpfor=0.0;

#ifdef CHECK
	check_off(p->parentage,p->offspring,p->n,1);
#endif
	
	wh=runiformint(1,p->n);						// choose one to split
	n1=runiformint(0,p->offspring[wh]);			// how many to move/keep ?

	lpfor = -LOG((double)p->n) - LOG((double)p->offspring[wh]+1.0)  - 
		lfactrl(p->offspring[wh]) + lfactrl(n1) + lfactrl(p->offspring[wh]-n1);

	change=nrivector0(1,p->offspring[wh]);		// set up a vector of those
	for (i=1;i<=n1;i++) change[i]=1;			// to change

	permute_integers(change,p->offspring[wh]);	// permute the vector
	
	for (count=0,i=1;i<=samples;i++) {			// change those wanted
		if (p->parentage[i]==wh) {
			count+=1;
			if (change[count]) p->parentage[i]=p->n+1;
		}
	}
	free_ivector(change,1);

	p->offspring[wh]-=n1;
	p->offspring[p->n+1]=n1;

	*lpback += M_LN2 - LOG((double)(p->n-p->fixed+1)) - LOG((double)p->n);	

	return lpfor + M_LN2;
}
/*************************************************************/
double splitparentageb(parents *p, double *lpback)
{
	int count=0,i,wh,*change,n1;
	double lpfor=0.0;
#ifdef CHECK
	check_off(p->parentage,p->offspring,p->n,3);
#endif
	
	for (count=0,i=1;i<=p->n;i++) if (p->offspring[i]>1) count++;  // how many are available
	if (count==0) myerror("none available to split!");		
	wh=runiformint(1,count);									   // choose a parent to split
	for (i=1;i<=p->n;i++) {
		if (p->offspring[i]>1) {
			if (wh==1) {
				wh=i;
				break;
			} else wh--;
		}
	}
	if (p->offspring[wh]==2) n1=1;									// how many for each split
	else n1=runiformint(1,p->offspring[wh]-1);

	lpfor = -LOG((double)count) - LOG((double)p->offspring[wh]-1.0)  - 
		lfactrl(p->offspring[wh]) + lfactrl(n1) + lfactrl(p->offspring[wh]-n1);

	change=nrivector0(1,p->offspring[wh]);
	for (i=1;i<=n1;i++) change[i]=1;
	permute_integers(change,p->offspring[wh]);

	for (count=0,i=1;i<=samples;i++) {
		if (p->parentage[i]==wh) {
			count+=1;
			if (change[count]) p->parentage[i]=p->n+1;
		}
	}
	free_ivector(change,1);
	p->offspring[wh]-=n1;
	p->offspring[(p->n)+1] = n1;
	*lpback += M_LN2 - LOG((double)(p->n-p->fixed)+1.) - LOG((double)(p->n));
#ifdef CHECK
	check_off(p->parentage,p->offspring,p->n+1,3);
#endif
	return lpfor + M_LN2;
}
/*************************************************************/
/*  Function to join two parents together                
    fixed safe												 */
double joinparentagea(parents *p,double *lpback)
{
	int wh,i;
	
	/*  select one to disappear  */
	wh=runiformint(p->fixed+1,p->n);  
	if (wh!=p->n) swap_parents(p,wh,p->n);
	wh = runiformint(1,p->n-1);
	
	*lpback += lfactrl(p->offspring[wh])+lfactrl(p->offspring[p->n]);
	/* now change offspring and parentage         */	
	p->offspring[wh]+=p->offspring[p->n];
	for (i=1;i<=samples;i++) {
		if (p->parentage[i]==p->n) 	p->parentage[i]=wh;
	}
	/* calculate the reverse probabilities   */
	*lpback += -LOG((double)p->n-1.0) -LOG((double)p->offspring[wh]+1.) 
		-lfactrl(p->offspring[wh]) + M_LN2; 
	return M_LN2 - LOG((double)(p->n-p->fixed)) - LOG((double)p->n-1.);
}
/*************************************************************/
double joinparentageb(parents *p,double *lpback)
{
	int wh,i,count=0;
	
	wh=runiformint(p->fixed+1,p->n);
	if (wh!=p->n) swap_parents(p,wh,p->n);
	wh = runiformint(1,p->n-1);

	*lpback += lfactrl(p->offspring[wh])+lfactrl(p->offspring[p->n]);
	/* now change offspring and parentage         */
	p->offspring[wh]+=p->offspring[p->n];
	for (i=1;i<=samples;i++) {
		if (p->parentage[i]==p->n) 	p->parentage[i]=wh;
	}
	/* calculate the reverse probabilities   */
	for (i=1;i<p->n;i++) if (p->offspring[i]>1) count++;

	*lpback += -LOG((double)count) -LOG((double)p->offspring[wh]-1.) -lfactrl(p->offspring[wh])
		+ M_LN2; 
	return M_LN2 - LOG((double)(p->n-p->fixed)) - LOG((double)p->n-1.);
}
/*******************************************************************/
double *lprob_parents(int *off,int **males,int nm, int **females,
					  int nf,int index,/* double *proploc,*/ int loci)
{
	int i,locus,j;
	double *p;
	
	p = nrdvector0(1,nm*nf);
	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=nm;i++) 
			for (j=1;j<=nf;j++) 
				p[nf*(i-1)+j] += log_lookup_prob(off+2*locus-2,males[i]+2*locus-2,
				females[j]+2*locus-2,index);//*proploc[locus];		
	}
	return p;
}
/*******************************************************************/
double *lprob_parent(int *off,int *other, int **pars,int n,
					 int index/*, double *proploc*/,int loci)
{
	int i,locus;
	double *lp;
	
	lp = nrdvector0(1,n);
	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=n;i++) 
			lp[i] += log_lookup_prob(off+2*locus-2,pars[i]+2*locus-2,
			other+2*locus-2,index);//*proploc[locus];
	}
	return lp;
}
/*******************************************************************/
void print_parents(int label, FILE *out, parents *p, char format)
{ 
	int i,j,**op,**ch,*off;
	char mess[40];


	if (p->fixed==p->range[1]&&p->nmissing==0) { // all the parents are known
		if (samples==1) {
			fprintf(out,"%d %d\n",label,p->parentage[1]);
		} else {
			off=nrivector0(1,p->n);
			for (i=1;i<=samples;i++) off[p->parentage[i]]++;
			for (i=1;i<=p->n;i++) {
				fprintf(out,"%d %d %d\n",label,i,off[i]);
			}
			free_ivector(off,1);
		}			
		return;
	}

	if (p->n==1) {
		sprintf(mess,"%d 1 %d ",label,samples);
		print_geno(out,mess,p->genotype[1],format);
		return;
	}

	if (p->fixed) {  //do not sort them 
		for (i=1;i<=p->n;i++) {
			sprintf(mess,"%d %d %d ",label,i,p->offspring[i]);
			print_geno(out,mess,p->genotype[i],format);
		}
		return;	
	}
	
	op=(int**)MALLOC((p->n+1)*sizeof(int*));
	if (!op) myerror("allocating op");
	ch=(int**)MALLOC((p->n+1)*sizeof(int*));
	if (!ch) myerror("allocating ch");
	off=nrivector0(1,p->n);
	for (i=1;i<=samples;i++) off[p->parentage[i]]++;
	for (i=1;i<=p->n;i++) {
		op[i]=copy_ivector(p->genotype[i],1,2*loci);
		ch[i]=op[i];
	}
	
	sort_genotypes(op,p->n,loci);
	for (i=1;i<=p->n;i++) {
		for (j=1;j<=p->n;j++) {
			if (ch[j]==op[i]) break;
		}
		sprintf(mess,"%d %d %d ",label,j,off[i]);
		print_geno(out,mess,op[i],format);
	}
	free_ivector(off,1);
	for (i=1;i<=p->n;i++)	free_ivector(op[i],1);
	FREE(ch);FREE(op);
	fflush(out);
	return;
}
/*********************************************************/
