/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#ifndef PARENTS_H_
#define PARENTS_H_
#include "offmodel.h"

/* The parents structure.
   This holds the details of one of the parents. */
typedef struct ptype parents;

struct ptype {
	int *known;		// a vector of known parents - note that this is 
					// only for fixed parents.  NULL if none known
	int n,fixed;	// number of parents and number fixed
	int range[2];	// min range[0] and max range[1] number of parents
	int *parentage; // vector of parents 
	int *offspring; // vector of numbner of sffspring that each parent has
	int **genotype; // genotype of the parents
	int *mssp,*mssl;// markers for missing data for known parents
	int nmissing;	// how many are missing
	double alpha;   // alpha - used for dirichlet and Ewens models
	offmodel off;   // where the model for parent offspring relations is kept.
	double (*splitparentage)(parents *, double *);  // functions for splitting 
	double (*joinparentage)(parents *, double *);	// and joining genotypes
	double (*gibbsparentaloffspring)(parents *, parents *,double,int);
};

double sample_n(parents *p);
double nparentsprior(parents *p);
/* Bookkeeping Functions */
parents copy_parents(parents *p);
void destroy_parents(parents *p);
int checkparent(parents *p);
void swap_parents(parents *p,int a, int b);
void copy_parentsinplace(parents *to,parents *from);
void print_parents(int label, FILE *out, parents *p,char format);

void write_parents(FILE *out, char *mess, parents *p);
parents read_parents(FILE *in,char *namestring);

parents assign_parents(prior pv,prior pa,int **pg, int np,int *range, int **Y);
void assign_offspring_random(parents *p1,parents *p2);
void resample_parents(parents *p,parents *ch);//, double *proploc);

double splitparentagea(parents *p, double *lpback);
double joinparentagea(parents *p, double *lpback);
double splitparentageb(parents *p, double *lpback);
double joinparentageb(parents *p, double *lpback);

double *lprob_parents(int *off,int **males,int nm, int **females,
					  int nf,int index/*, double *proploc*/, int loci);
double *lprob_parent(int *off,int *other, int **pars,int n,int index/*, double *proploc*/, int loci);

#endif
