/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include "stdio.h"
#include "stdlib.h"
#include "myio.h"
#include "random.h"
#include "nrutil.h"
#include "util.h"
#include "patlookup.h"
#include "patutil.h"
#include "mcmc_paternity.h"
#include "prior.h"
#include "randparent.h"
#include "model.h"
#include "parameters.h"
#include "data.h"
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "output.h"

double par[3]={100,1,1};
/******************************************************************/
/******************************************************************/
//#define ALLOUTPUT  //if this is set gives extended output
chain *outputlinetemper(FILE *out, int which,chain *c,int tempering);
void outputline(FILE *out, chain *use);
int main(int argc, char *argv[])
{
	model m;
	output out;
	int i,j,k,wh,indx,nfuncs,run,reps;	
	char infile[40]="infile",outfile[40]="outfile";
	FILE *OUT_FILE,*FEMALES_FILE,*MALES_FILE;
	chain *use=NULL;
	int seed=0;

	printf("Paternity\n---------\n\n"
		   "Getting Input\n-------------\n\n");

	if (argc==1) {
		printf("using paternity infile outfile\n");
	} else if (argc>=3) {
		strcpy(infile,argv[1]);strcpy(outfile,argv[2]);
		if (argc==4) seed = atoi(argv[3]);
	} else {
		printf("use paternity\nor paterntity infile outfile (seed)\n");	
		exit(0);
	}

	m.param=readparameters(infile);
	starTup((seed>0)?seed:m.param.seed);
	m.dat=read_data(&m.param);
	starting_chains(&m);
	nfuncs=choose_gibbsmets(&m);	
	out=create_output(m.dat.samples,m.dat.loci);

	OUT_FILE = openoutputfile(outfile);
	MALES_FILE=openstemleafoutputfile(outfile,"males");
	FEMALES_FILE=openstemleafoutputfile(outfile,"females");

#ifdef ALLOUTPUT
	if (!m.tempering) outputline(OUT_FILE,use = &m.c[1]);
	else {
		use = outputlinetemper(OUT_FILE,1,m.c,m.tempering);
		for (k=2;k<=m.tempering+1;k++) {
			outputlinetemper(OUT_FILE,k,m.c,m.tempering);
		}
	}
#endif
	
	if (m.param.warmup>0) {
		reps=m.param.warmup;run=0;
		printf("warming up\n----------\n");
	} else {
		reps=m.param.replicates;run=1;
	}
	for (i=1;i<=reps;i++) {
		for (j=1;j<=m.param.between;j++) {
			for (k=1;k<=m.tempering+1;k++) {		
#ifdef CHATTER
				printf("%d %d %d\n",i,j,k);fflush(stdout);
#endif 
#ifdef CHECK
				check_off(m.c[k].fathers.parentage,m.c[k].fathers.offspring,m.c[k].fathers.n,
					m.c[k].fathers.off.model);
				check_off(m.c[k].mothers.parentage,m.c[k].mothers.offspring,m.c[k].mothers.n,
					m.c[k].mothers.off.model);
#endif
				indx=m.c[k].temperindex;
				wh=runiformint(0,nfuncs-1);
				
				if (wh<m.ngibbs) m.gibbsfuncs[wh](&m.c[k]);
				else {
					wh-=m.ngibbs;
					if (wh<m.nmets)  {
						m.success[indx][wh] += m.metrofuncs[wh](&m.c[k],m.attempt[indx]+wh);
					} else {			
						indx=runiformint(1,m.tempering+m.param.independence);	
						if (indx==m.tempering+1) 
							m.success[indx][wh]+=metro_newsample(&m,indx,m.attempt[indx]+wh);
						else 
							m.success[indx][wh]+=metro_temper(&m,indx,m.attempt[indx]+wh);		
					}			
				}
			}
		}
		if (i%20==0) {				
			printf(".");				
			if (i%1000==0) printf("\n");
			fflush(stdout);
		}
		if (i==reps&&run==0) {
			printf("\ntaking samples\n--------------\n");
			reps=m.param.replicates;run=1;
			for (i=1;i<=m.tempering+1;i++) {
				for (j=0;j<=6;j++) {
					m.attempt[i][j]=0;m.success[i][j]=0;
				}
			}
			i=1;
		}
		if (run) { //take samples
#ifdef ALLOUTPUT
			if (!m.tempering) outputline(OUT_FILE,use = &m.c[1]);
			else {
				use = outputlinetemper(OUT_FILE,1,m.c,m.tempering);
				for (k=2;k<=m.tempering+1;k++) {
					outputlinetemper(OUT_FILE,k,m.c,m.tempering);
				}
			}
#else
			if (!m.tempering) {
				outputline(OUT_FILE,use=&m.c[1]);
			} else use = outputlinetemper(OUT_FILE,1,m.c,m.tempering);
#endif
			update_output(&out,use->fathers.parentage,use->mothers.parentage,
				m.dat.Y,use->fathers.genotype,use->mothers.genotype);
			
			print_parents(i,MALES_FILE,&use->fathers,m.param.format);
			print_parents(i,FEMALES_FILE,&use->mothers,m.param.format);		
		}
	}
	printf("\noutput\n------\n\n");
    fclose(MALES_FILE);fclose(FEMALES_FILE);fclose(OUT_FILE);
	/*  Now write the output and destroy any memory used  */
	
	write_output(outfile,&out);

	printparameters(OUT_FILE=openstemleafoutputfile(outfile,"par"),&m.param);
	printallrates(stdout,&m);
	printallrates(OUT_FILE,&m);
	fclose(OUT_FILE);
	
	write_chain(openstemleafoutputfile(outfile,"end"),use);
	write_sorteddata(openstemleafoutputfile(outfile,"clean")
		,&m.dat,use,1);
	/*  Now free all the memory allocated */
	destroy_output(&out);	
	destroy_lookup();
	destroy_model(&m);
	
	return 0;
}
/**********************************************************/
chain *outputlinetemper(FILE *out, int which,chain *c, int tempering)
{
	int i;
	chain *use=NULL;
	
    for (i=1;i<=tempering+1;i++) { 
		if (c[i].temperindex==which) {
			use=&c[i];
			break;
		}
	}
	if (use==NULL)
		myerror("not found correct chain in outputlinetemper");
#ifdef ALLOUTPUT
	for (i=0;i<=use->loci;i++) 
		fprintf(out,"%g ",use->proploc[i]);
#endif
	outputline(out,use);
	
	return use;
}
/**********************************************************/
void outputline(FILE *out, chain *use)
{	
		fprintf(out,"%d %d %d %g %g %g",
			use->fathers.n,use->mothers.n,
			count_muts(&(use->fathers),&(use->mothers)),use->mu,
			use->fathers.alpha,use->mothers.alpha);
		print_probs(out,use);
		fprintf(out,"\n");
#ifdef ALLOUTPUT
	fflush(out);
#endif
}
/**********************************************************************/
