/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <math.h>
#include "nrutil.h"
#include "util.h"
#include "random.h"
#include "patlookup.h"
#include "patutil.h"
#include "prior.h"
#include "myio.h"
/*******************************************************************/
/*******************************************************************/
extern int **Y,loci,*alleles,samples,**allele_labels,max_alleles,freqmodel;
extern double **rho;
/*******************************************************************/
/*******************************************************************/
prior unipr={0,{1.0,0}};
/*******************************************************************/
double sample_n(parents *p)
{
	double x;
	
	if (p->off.model==1) {
		if (p->off.nprior.prtype==CONSTPRIOR) {
			p->n=(int)(p->off.nprior.par[0]+0.5);
			return 0.0;
		}
		for (;;) {
			sample_prior(&x,p->off.nprior);
			p->n=(int)(x+0.5);
			if (p->n>p->range[0]&&p->n<=p->range[1]) break;
		}
		return log_prior(&x,p->off.nprior);
	}
	else {
		p->n=runiformint(p->range[0],p->range[1]);
		return 0.0;
	}
}
/*******************************************************************/
double lpgenolocus(int *a, double *rh, int k, double temper)
{
	if (freqmodel) {
		if (a[1]==a[2]) 
			return 2.*log(rh[a[1]])/temper;
		else return (log(rh[a[1]])+log(rh[a[2]]))/temper;
	}
	else return log_dmulti_dirichletctemper(a,rh,k,2,temper);
}
/*******************************************************************/
double log_like(int **fgenotype, int *fathers,int **mgenotype, 
				int *mothers, int index)//, double *proploc)
{
	int i,locus;
	double lp=0.0;

	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=samples;i++) {
			lp += log_lookup_prob(Y[i]+2*locus-2,
				fgenotype[fathers[i]]+2*locus-2,
				mgenotype[mothers[i]]+2*locus-2,index);//*proploc[locus];
		}
	}
	return lp;
}
/******************************************************************/
double log_parents(int **fgenotype,  int nfathers,
				   int **mgenotype,  int nmothers,
				   double temper)
{
	int i,locus;
	double lp=0.0;
	for (locus=1;locus<=loci;locus++) {
		if (freqmodel) {
			for (i=1;i<=nfathers;i++) {
				if (fgenotype[i][2*locus-1]>0)
				lp += log_multinomial3temper(
					fgenotype[i]+2*locus-2,rho[locus],alleles[locus],2,temper/*[locus]*/);
			}
			for (i=1;i<=nmothers;i++) {
				if (mgenotype[i][2*locus-1]>0)
				lp += log_multinomial3temper(
					mgenotype[i]+2*locus-2,rho[locus],alleles[locus],2,temper/*[locus]*/);
			}
		} else {
			for (i=1;i<=nfathers;i++) {
				if (fgenotype[i][2*locus-1]>0)
				lp += log_dmulti_dirichletctemper(
					fgenotype[i]+2*locus-2,rho[locus],alleles[locus],2,temper/*[locus]*/);
			}
			for (i=1;i<=nmothers;i++) {
				if (mgenotype[i][2*locus-1]>0)
				lp += log_dmulti_dirichletctemper(
					mgenotype[i]+2*locus-2,rho[locus],alleles[locus],2,temper/*[locus]*/);
			}
		}
	}
	return lp;
}
/******************************************************************/
double log_priors(parents *males, parents *females, double mu, 
				  prior muprior, double temper3)
{
	double lp;

	lp = lpost_off(males->offspring,males->n,males->alpha,&males->off,temper3);
	lp += lpost_off(females->offspring,females->n,females->alpha,&females->off,temper3);
    lp+=log_prior(&mu,muprior)/temper3;
	
	return lp;
}
/******************************************************************/
void print_probs(FILE *out, chain *c)
{
	fprintf(out," %g %g %g %g ",log_posterior(c,c->temper,c->temper3),//,c->proploc),
		log_like(c->fathers.genotype,c->fathers.parentage,
		c->mothers.genotype,c->mothers.parentage,c->muindex/*,c->proploc*/)/c->temper,
		log_parents(c->fathers.genotype,c->fathers.n,
		c->mothers.genotype,c->mothers.n,c->temper3),//,c->proploc),
		log_priors(&c->fathers,&c->mothers,c->mu,c->muprior,c->temper3));
	
}
/******************************************************************/
double log_posterior(chain *c,double temper,
					 double temper3)//, double *proploc)
{
	return log_like(c->fathers.genotype,c->fathers.parentage,
		c->mothers.genotype,c->mothers.parentage,c->muindex/*,proploc*/)/temper +
		log_parents(c->fathers.genotype,c->fathers.n,
		c->mothers.genotype,c->mothers.n,c->temper3/*,proploc*/)+
		log_priors(&c->fathers,&c->mothers,c->mu,c->muprior,temper3);
}
/*******************************************************************/
/*  a utility function to give the row and column number for a 
sequentially numbers position on an upper matrix -there 
must be a better way !                                             */
/*******************************************************************/
void gens(int *a,int num, int l)
{
	int i;
	for (i=1;i<=l;i++) {
		if (num<=l-i+1) {
			a[1]=i;
			a[2]=i+num-1;
			return;
		}
		num-=l-i+1;
	}
}
/*   The inverse function   */
int getnum(int *x, int l)
{
	return x[2]+(x[1]-1)*(2*l-x[1])/2;
}
/***********************************************************************/
/*   Fixed safe                                                        */
double gibbsparentalgenotypes(parents *ch,parents *st,double temper,double temper3,
									  int index)//, double *proploc)
{
	int locus,count,ma,a[3],nf,i,lena;
	double **pr,*p,lpselect=0.0,psel,mp;

	if (ch->n==ch->fixed) {
		return 0.0;
	}
#ifdef CHECK
	check_off(ch->parentage,ch->offspring,ch->n,ch->off.model);
	check_off(st->parentage,st->offspring,st->n,st->off.model);
#endif
	
	ma=max_alleles*(max_alleles+1)/2;
	pr=nrdmatrix(ch->fixed+1,ch->n,1,ma);
	p=nrdvector(1,ma);
	
	for (locus=1;locus<=loci;locus++) {
		lena=alleles[locus]*(alleles[locus]+1)/2;
		for (count=ch->fixed+1;count<=ch->n;count++) {
			for (i=1;i<=lena;i++) pr[count][i]=0.0; 
		}
		for (count=1,a[1]=1;a[1]<=alleles[locus];a[1]++) {
			for (a[2]=a[1];a[2]<=alleles[locus];a[2]++,count++) {
				for (i=1;i<=samples;i++) {
					if (ch->parentage[i]>ch->fixed)
						pr[ch->parentage[i]][count] +=log_lookup_prob(Y[i]+2*locus-2,a,
						st->genotype[st->parentage[i]]+2*locus-2,index);//*proploc[locus];
				}
			}
		}
		for (i=ch->fixed+1;i<=ch->n;i++) {	
			mp=dmax(pr[i],lena);
			for (count=1,a[1]=1;a[1]<=alleles[locus];a[1]++) {
				for (a[2]=a[1];a[2]<=alleles[locus];a[2]++,count++) {
					p[count]=exp(-mp+ pr[i][count]/temper + 
						lpgenolocus(a,rho[locus],alleles[locus],temper3)
						);
				}
			}
			nf=gen_from_probs2(p,lena,&psel);
			lpselect += log(psel);
			gens(ch->genotype[i]+2*locus-2,nf,alleles[locus]);
		}
	}	
	free_dvector(p,1);
	free_dmatrix(pr,ch->fixed+1,1);
	return lpselect;
}
/***********************************************************************/
void gibbsonelocus(int *g, parents *st,double temper,double temper3,
				   int index, int *parent, int whpar,int loc,int als)
{
	int count,ma,nf,a[3],lena,i;
	double *pr;
	
	ma=max_alleles*(max_alleles+1)/2;
	pr=nrdvector0(1,ma);
	
	lena=als*(als+1)/2;
	
	for (count=1,a[1]=1;a[1]<=als;a[1]++) {
		for (a[2]=a[1];a[2]<=als;a[2]++,count++) {	
			for (i=1;i<=samples;i++) {
				if (parent[i]==whpar) {
					pr[count] += log_lookup_prob(Y[i]+2*loc-2,a,
						st->genotype[st->parentage[i]]+2*loc-2,index);
				}
			}
			pr[count] = exp(pr[count]/temper +	lpgenolocus(a,rho[loc],als,temper3));
		}
	}
	nf=gen_from_probs(pr,lena);
	gens(g,nf,als);	
	free_dvector(pr,1);	
}
/***********************************************************************/
/*  function to calculate the probability of sampling 
parents ch

  safe for use with fixed parents
  safe for use with known parents
  
*/
/***********************************************************************/
double lpselectedparentalgenotypes(parents *ch,parents *st,
								   double temper, double temper3, int index)//, double *proploc)
{
	int locus,count,ma,a[3],nf,i,lena;
	double **pr,*p,lpselect=0.0;
	
	if (ch->n==ch->fixed) return 0.0;
	
	ma=max_alleles*(max_alleles+1)/2;
	pr=nrdmatrix(ch->fixed+1,ch->n,1,ma);
    p=nrdvector(1,ma);
	
	for (locus=1;locus<=loci;locus++) {
		lena=alleles[locus]*(alleles[locus]+1)/2;
		for (count=ch->fixed+1;count<=ch->n;count++) {
			for (i=1;i<=lena;i++) pr[count][i]=0.0;  /* reset pr to 0*/
		}
		for (count=1,a[1]=1;a[1]<=alleles[locus];a[1]++) {
			for (a[2]=a[1];a[2]<=alleles[locus];a[2]++,count++) {
				for (i=1;i<=samples;i++) {
					if (ch->parentage[i]>ch->fixed)
						pr[ch->parentage[i]][count] +=log_lookup_prob(Y[i]+2*locus-2,a,
						st->genotype[st->parentage[i]]+2*locus-2,index);//*proploc[locus];
				}
			}
		}
		
		for (i=ch->fixed+1;i<=ch->n;i++) {
			for (count=1,a[1]=1;a[1]<=alleles[locus];a[1]++) {
				for (a[2]=a[1];a[2]<=alleles[locus];a[2]++,count++) {
					p[count]=exp(pr[i][count]/temper + 
						lpgenolocus(a,rho[locus],alleles[locus],temper3)
						);
				}
			}
			nf = getnum(ch->genotype[i]+2*locus-2,alleles[locus]);
			lpselect += log(get_prob(nf,p,lena));
		}
	}	
	free_dvector(p,1);
	free_dmatrix(pr,ch->fixed+1,1);
	return lpselect;
}
/**************************************************************************/
/*  This function changes the parentage and offspring vectors of          */
/*  parents *ch (for change)											  */
/*  by sampling from the conditional distribution of parentage given      */
/*  the genotype data - and all other information                         */
/**************************************************************************/
double sampleparentaloffspring(parents *ch, parents *st,
							 double temper,int index)//, double *proploc)
{
	int off,i;
	double *pr,lp=0.0,p;
	
	if (ch->n==1) return 0.0;			/*  if there is only one parent nothing to do	*/
	if (ch->known && ch->known[0]==samples)	return 0.0; /* if we know them all we are finished  */
	for (i=1;i<=ch->n;i++) ch->offspring[i]=0;
	if (ch->known) {					/* do we know any of the parentages?			*/ 
		for (off=1;off<=samples;off++) {
			if (!ch->known[off]) {		/* if we do not know the parentage of i then sample*/
				pr = lprob_parent(Y[off],st->genotype[st->parentage[off]],
					ch->genotype,ch->n,index,/*proploc,*/loci);
				for (i=1;i<=ch->n;i++) {
					pr[i] = exp(pr[i]/temper);
				}
				ch->parentage[off]=gen_from_probs2(pr,ch->n,&p);
				lp+=log(p);
				free_dvector(pr,1);
			} 	
			ch->offspring[ch->parentage[i]]+=1;
		}
	} else {
		for (off=1;off<=samples;off++) {
			pr = lprob_parent(Y[off],st->genotype[st->parentage[off]],
				ch->genotype,ch->n,index/*,proploc*/,loci);
			for (i=1;i<=ch->n;i++) {
				pr[i] = exp(pr[i]/temper);
			}
			ch->parentage[off]=gen_from_probs2(pr,ch->n,&p);
			lp+=log(p);
			ch->offspring[ch->parentage[i]]+=1;
			free_dvector(pr,1);
		}
	}	
	return lp;
}
/**************************************************************************/
double gibbsparentaloffspringa(parents *ch, parents *st,
							  double temper,int index)//, double *proploc)
{
	int off,i;
	double *pr,lp=0.0,p;
	
	if (ch->n==1) return 0.0;			/*  if there is only one parent nothing to do	*/
	if (ch->known && ch->known[0]==samples)	return 0.0; /* if we know them all we are finished  */
	for (i=1;i<=ch->n;i++) ch->offspring[i]=0;
	if (ch->known) {					/* do we know any of the parentages?			*/ 
		for (off=1;off<=samples;off++) {
			if (!ch->known[off]) {		/* if we do not know the parentage of i then sample*/
				pr = lprob_parent(Y[off],st->genotype[st->parentage[off]],
					ch->genotype,ch->n,index/*,proploc*/,loci);
				switch (ch->off.model) {
				case 1:
					for (i=1;i<=ch->n;i++) pr[i] = exp(pr[i]/temper);
					break;
				case 2:
					for (i=1;i<=ch->n;i++) 
						pr[i] = exp(pr[i]/temper)*
						(ch->alpha+(double)ch->offspring[i])/(ch->alpha+(double)off);
					break;
				case 3:
					for (i=1;i<=ch->n;i++) {
						if (ch->offspring[i]==0) 
							pr[i]=exp(pr[i]/temper)*ch->alpha/(ch->alpha+(double)(off-1));
						else 
							pr[i]=exp(pr[i]/temper)*
							(double)ch->offspring[i]/(ch->alpha+(double)(off-1));
					}
					break;
				}
				ch->parentage[off]=gen_from_probs2(pr,ch->n,&p);
				lp+=log(p);
				free_dvector(pr,1);
			} 	
			ch->offspring[ch->parentage[i]]+=1;
		}
	} else {
		for (off=1;off<=samples;off++) {
			pr = lprob_parent(Y[off],st->genotype[st->parentage[off]],
				ch->genotype,ch->n,index/*,proploc*/,loci);
			switch (ch->off.model) {
			case 1:
				for (i=1;i<=ch->n;i++) pr[i] = exp(pr[i]/temper);
				break;
			case 2:
				for (i=1;i<=ch->n;i++) 
					pr[i] = exp(pr[i]/temper)*
					(ch->alpha+(double)ch->offspring[i])/(ch->alpha+(double)off);
				break;
			case 3:
				for (i=1;i<=ch->n;i++) {
					if (ch->offspring[i]==0) 
						pr[i]=exp(pr[i]/temper)*ch->alpha/(ch->alpha+(double)(off-1));
					else 
						pr[i]=exp(pr[i]/temper)*
						(double)ch->offspring[i]/(ch->alpha+(double)(off-1));
				}
				break;
			}	
			ch->parentage[off]=gen_from_probs2(pr,ch->n,&p);
			if (ch->parentage[off]>ch->n||ch->parentage[off]<1)
				myerror("error sampling parentage XX1");
			lp+=log(p);
			ch->offspring[ch->parentage[off]]+=1;
			free_dvector(pr,1);
		}
	}	
	return lp;
}
/**************************************************************************/
/*  Another gibbs - this time conditioning on one of each of the offspring
    being from each of the parents - keeping constant from before         */
/**************************************************************************/
double gibbsparentaloffspringb(parents *ch, parents *st,
							  double temper,int index)//, double *proploc)
{
	int off,i,*keep;
	double *pr,lp=0.0,p;
	
#ifdef CHECK
	check_off(ch->parentage,ch->offspring,ch->n,ch->off.model);
	check_off(st->parentage,st->offspring,st->n,st->off.model);
#endif
	
	if (ch->n==1) return 0.0;			/*  if there is only one parent nothing to do	*/
	if (ch->known && ch->known[0]==samples)	return 0.0; /* if we know them all we are finished  */
	
	keep=nrivector(1,ch->n);
	for (i=1;i<=ch->n;i++)  {
		keep[i]=runiformint(1,ch->offspring[i]);
		ch->offspring[i]=0;
	}
	if (ch->known) {					/* do we know any of the parentages?			*/ 
		for (off=1;off<=samples;off++) {  /* note that this is wrong - the known are not incorporated  */
			keep[ch->parentage[off]-=1];
			if (!ch->known[off]&&keep[ch->parentage[off]]!=0) {		/* if we do not know the parentage of i then sample*/
				pr = lprob_parent(Y[off],st->genotype[st->parentage[off]],
					ch->genotype,ch->n,index,/*proploc,*/loci);
				switch (ch->off.model) {
				case 1:
					for (i=1;i<=ch->n;i++) pr[i] = exp(pr[i]/temper);
					break;
				case 2:
					for (i=1;i<=ch->n;i++) 
						pr[i] = exp(pr[i]/temper)*
						(ch->alpha+(double)ch->offspring[i])/(ch->alpha+(double)off);
					break;
				case 3:
					for (i=1;i<=ch->n;i++) {
						if (ch->offspring[i]==0) 
							pr[i]=exp(pr[i]/temper)*ch->alpha/(ch->alpha+(double)(off-1));
						else 
							pr[i]=exp(pr[i]/temper)*
							(double)ch->offspring[i]/(ch->alpha+(double)(off-1));
					}
					break;
				}
				ch->parentage[off]=gen_from_probs2(pr,ch->n,&p);
				ch->offspring[ch->parentage[i]]+=1;
				lp+=log(p);
				free_dvector(pr,1);
			} 			
		}
	} else {
		for (off=1;off<=samples;off++) {
			keep[ch->parentage[off]]-=1;
			if (keep[ch->parentage[off]]!=0) {
				pr = lprob_parent(Y[off],st->genotype[st->parentage[off]],
					ch->genotype,ch->n,index/*,proploc*/,loci);
				switch (ch->off.model) {
				case 1:
					for (i=1;i<=ch->n;i++) pr[i] = exp(pr[i]/temper);
					break;
				case 2:
					for (i=1;i<=ch->n;i++) 
						pr[i] = exp(pr[i]/temper)*
						(ch->alpha+(double)ch->offspring[i])/(ch->alpha+(double)off);
					break;
				case 3:
					for (i=1;i<=ch->n;i++) {
						if (ch->offspring[i]==0) 
							pr[i]=exp(pr[i]/temper)*ch->alpha/(ch->alpha+(double)(off-1));
						else 
							pr[i]=exp(pr[i]/temper)*
							(double)ch->offspring[i]/(ch->alpha+(double)(off-1));
					}
					break;
				}	
				ch->parentage[off]=gen_from_probs2(pr,ch->n,&p);
				if (ch->parentage[off]>ch->n||ch->parentage[off]<1)
					myerror("error sampling parentage XX1");
				lp+=log(p);
				free_dvector(pr,1);
			}		
			ch->offspring[ch->parentage[off]]+=1;	
		}	
	}
	free_ivector(keep,1);
	return lp;
}
/*******************************************************************/
double sampleallparentaloffspringa(parents *ch, parents *st,double temper, 
								  int index)//, double *proploc)
{
	int off,i,val;
	double *pr,*pr2,prob,lp=0.0;
	
	if (ch->n==1&&st->n==1) {
		for (i=1;i<=samples;i++) {
			ch->parentage[i]=1;
			st->parentage[i]=1;
		}
		return 0.0;
	}
	
	for (i=1;i<=ch->n;i++) ch->offspring[i]=0;
	for (i=1;i<=st->n;i++) st->offspring[i]=0;
	
	for (off=1;off<=samples;off++) {
		pr = lprob_parents(Y[off],ch->genotype,ch->n,
			st->genotype,st->n,index/*,proploc*/,loci);
		pr2= lproboff(ch->offspring,ch->n,ch->alpha,&ch->off,
			st->offspring,st->n,st->alpha,&st->off,off);
		for (i=1;i<=ch->n*st->n;i++) pr[i]=exp(pr2[i]+pr[i]/temper);
		val=gen_from_probs2(pr,ch->n*st->n,&prob);
		lp += log(prob);
		ch->parentage[off]=(val-1)/st->n+1;
		if (ch->parentage[off]<1||ch->parentage[off]>ch->n)
			myerror("error allocating parentage 1");
		ch->offspring[ch->parentage[off]]+=1;
		st->parentage[off]=(val-1)%st->n+1;
		if (st->parentage[off]<1||st->parentage[off]>st->n)
			myerror("error allocating parentage 1");
		st->offspring[st->parentage[off]]+=1;
		free_dvector(pr2,1);
		free_dvector(pr,1);
	}
	return lp;
}
/*******************************************************************/
double sampleallparentaloffspringb(parents *ch, parents *st,double temper, 
								   int index)//, double *proploc)
{
	int off,i,val,*keepch,*keepst;
	double *pr,*pr2,prob,lp=0.0;
	
	if (st->n==1) {
		if (ch->n!=1) 
			return gibbsparentaloffspringb(ch,st,temper,index);//,proploc);
		else return 0.0;
	} else if (ch->n==1) 
		return gibbsparentaloffspringb(st,ch,temper,index);//,proploc);
	
	keepch=nrivector(1,ch->n);
	keepst=nrivector(1,st->n);

	for (i=1;i<=ch->n;i++) {
		keepch[i]=runiformint(1,ch->offspring[i]);
		ch->offspring[i]=1;
	}
	
	for (i=1;i<=st->n;i++) {
		keepst[i]=runiformint(1,st->offspring[i]);
		st->offspring[i]=1;
	}
	
	for (off=1;off<=samples;off++) {
		keepch[ch->parentage[off]]-=1;
		keepst[st->parentage[off]]-=1;
		if (keepch[ch->parentage[off]]==0) {
			if (keepst[st->parentage[off]]!=0) {
				pr = lprob_parent(Y[off],ch->genotype[ch->parentage[off]],
					st->genotype,st->n,index/*,proploc*/,loci);
				switch (ch->off.model) {
				case 1:
					for (i=1;i<=st->n;i++) pr[i] = exp(pr[i]/temper);
					break;
				case 2:
					for (i=1;i<=st->n;i++) 
						pr[i] = exp(pr[i]/temper)*
						(st->alpha+(double)st->offspring[i])/(st->alpha+(double)off);
					break;
				case 3:
					for (i=1;i<=st->n;i++) {
						if (st->offspring[i]==0) 
							pr[i]=exp(pr[i]/temper)*st->alpha/(st->alpha+(double)(off-1));
						else 
							pr[i]=exp(pr[i]/temper)*
							(double)st->offspring[i]/(st->alpha+(double)(off-1));
					}
					break;
				}	
				st->parentage[off]=gen_from_probs2(pr,st->n,&prob);
				st->offspring[st->parentage[off]]+=1;
				if (st->parentage[off]>st->n||ch->parentage[off]<1)
					myerror("error sampling parentage XX1");
				lp+=log(prob);
				free_dvector(pr,1);
			}
		} else if (keepst[st->parentage[off]]==0) {
			pr = lprob_parent(Y[off],st->genotype[st->parentage[off]],
				ch->genotype,ch->n,index/*,proploc*/,loci);
			switch (ch->off.model) {
			case 1:
				for (i=1;i<=ch->n;i++) pr[i] = exp(pr[i]/temper);
				break;
			case 2:
				for (i=1;i<=ch->n;i++) 
					pr[i] = exp(pr[i]/temper)*
					(ch->alpha+(double)ch->offspring[i])/(ch->alpha+(double)off);
				break;
			case 3:
				for (i=1;i<=ch->n;i++) {
					if (ch->offspring[i]==0) 
						pr[i]=exp(pr[i]/temper)*ch->alpha/(ch->alpha+(double)(off-1));
					else 
						pr[i]=exp(pr[i]/temper)*
						(double)ch->offspring[i]/(ch->alpha+(double)(off-1));
				}
				break;
			}	
			ch->parentage[off]=gen_from_probs2(pr,ch->n,&prob);
			ch->offspring[ch->parentage[off]]+=1;
			if (ch->parentage[off]>ch->n||ch->parentage[off]<1)
				myerror("error sampling parentage XX1");
			lp+=log(prob);
			free_dvector(pr,1);
		} else {
			pr = lprob_parents(Y[off],ch->genotype,ch->n,
				st->genotype,st->n,index/*,proploc*/,loci);
			pr2= lproboff(ch->offspring,ch->n,ch->alpha,&ch->off,
				st->offspring,st->n,st->alpha,&st->off,off);
			for (i=1;i<=ch->n*st->n;i++) 
				pr[i]=exp(pr2[i]+pr[i]/temper);
			val=gen_from_probs2(pr,ch->n*st->n,&prob);
			lp += log(prob);
			ch->parentage[off]=(val-1)/st->n+1;
			if (ch->parentage[off]<1||ch->parentage[off]>ch->n)
				myerror("error allocating parentage 1");
			st->parentage[off]=(val-1)%st->n+1;
			if (st->parentage[off]<1||st->parentage[off]>st->n)
				myerror("error allocating parentage 1");
			free_dvector(pr2,1);
			free_dvector(pr,1);	
			ch->offspring[ch->parentage[off]]+=1;		
	    	st->offspring[st->parentage[off]]+=1;
		}
	}
	free_ivector(keepch,1);
	free_ivector(keepst,1);
	return lp;
}
/*******************************************************************/
double lpsampleallparentaloffspring(parents *ch, parents *st,double temper, 
									int index)//, double *proploc)
{
	int off,i,val;
	double *pr,lp=0.0;
	
	if (ch->n==1&&st->n==1) return 0.0;
	
	for (off=1;off<=samples;off++) {
		pr = lprob_parents(Y[off],ch->genotype,ch->n,
			st->genotype,st->n,index/*,proploc*/,loci);
		for (i=1;i<=ch->n*st->n;i++) pr[i]=exp(pr[i]/temper);
		val = (ch->parentage[off]-1)*st->n+(st->parentage[off]);
		lp += log(get_prob(val,pr,ch->n*st->n));
		free_dvector(pr,1);
	}
	return lp;
}
/******************************************************************/
int count_muts(parents *m, parents *f)
{
	int locus,i,count=0;
	
	for (locus=1;locus<=loci;locus++) {
		for (i=1;i<=samples;i++) {
			count+=lookup_muts(Y[i]+2*locus-2,
				m->genotype[m->parentage[i]]+2*locus-2,
				f->genotype[f->parentage[i]]+2*locus-2);
		}
	}
	return count;
}
/******************************************************************/

