/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <math.h>
#include "nrutil.h"
#include "util.h"
#include "random.h"
#include "patlookup.h"
#include "patutil.h"
#include "randparent.h"
#include "prior.h"
#include "myio.h"
/****************************************************************/
extern int samples,loci,**Y,*alleles,max_alleles,**allele_labels;
/****************************************************************/
double **freq=NULL;
void get_freq(void)
{
	int locus,i;
	double sum,se;
	
	freq=nrdmatrix0(1,loci,1,max_alleles);		
	for (i=1;i<=samples;i++) {
		for (locus=1;locus<=loci;locus++) {
			freq[locus][Y[i][2*locus-1]]+=1.0/(2.*samples);
			freq[locus][Y[i][2*locus]]+=1.0/(2.*samples);
		}
	}
	se = sqrt(0.25*.75/(2.*samples));
	for (locus=1;locus<=loci;locus++) {
		sum=0;
		for (i=1;i<=max_alleles;i++) {
			if (freq[locus][i]<0.25-2.5*se)
				freq[locus][i] /=100.;
			sum += freq[locus][i];
		}
		for (i=1;i<=max_alleles;i++) {
			freq[locus][i]/=sum;
		}
	}
}
/****************************************************************/
double samplemother(int **genotype)
{
	int locus,g;
	double lp=0.0,p,sub;
	if (freq==NULL) get_freq();
	for (locus=1;locus<=loci;locus++) {
		g=gen_from_probs2(freq[locus],alleles[locus],&p);
		lp+=log(p);
		sub=freq[locus][g];
		if (freq[locus][g]>0.45) freq[locus][g] -=0.45;
		else freq[locus][g]=0.01;
		genotype[1][2*locus-1]=g;
		genotype[1][2*locus]=gen_from_probs2(freq[locus],alleles[locus],&p);
		lp+=log(p);
		freq[locus][g]=sub;
	}
	return lp;
}
/****************************************************************/
double lpsamplemother(int **genotype)
{
	int locus,g;
	double lp=0.0,sub;
	if (freq==NULL) get_freq();
	for (locus=1;locus<=loci;locus++) {
		g = genotype[1][2*locus-1];
		lp+= log(get_prob(g,freq[locus],alleles[locus]));
		sub=freq[locus][g];
		if (freq[locus][g]>0.45) freq[locus][g] -=0.45;
		else freq[locus][g]=0.01;
		lp+= log(get_prob(genotype[1][2*locus],freq[locus],alleles[locus]));
		freq[locus][g]=sub;
	}
	return lp;
}
void free_freq(void)
{
	free_dmatrix(freq,1,1);
}
extern double par[3];
/*******************************************************************/
double glookup(int *g, int *Y, int *m, int l)
{
	int add=0;
	double d;
	
	if (Y[1]==Y[2]) {
		add=Y[1];
	} else {
		if (Y[1]!=m[1]&&Y[1]!=m[2]) add= Y[1];
		else if (Y[2]!=m[1]&&Y[2]!=m[2]) add= Y[2];
	}
	
	if (add>0) {
		if (g[1]&&g[2]) {  /* the genotype filled up */
			if (add!=g[1]&&add!=g[2]) d= par[0]; //a mismatch
			else d= 0.;    // match
		} 
		else if (g[1]>0) { /* space to add one allele */
			if (add!=g[1]) return par[1]*alleles[l];//a partial mismatch - what do we do?
			else d= 0.;
		}
		else d= par[2]*alleles[l]*alleles[1];  //  full space
	} else {
		if (g[1]&&g[2]) {
			if ((Y[1]!=g[1]||Y[2]!=g[2])||
				(Y[1]!=g[2]||Y[2]!=g[1])) d=par[0]; //a mismatch
			else d= 0;
		} else if (g[1]) {
			if (Y[1]!=g[1]&&Y[2]!=g[1])
				d= par[1]*alleles[l];//a partial mismatch - what do we do?
			else d= 0.;
		} else d= par[2]*alleles[l]*alleles[1];  //  full space
	}
	return d;
}
/*******************************************************************/
void getparprobs(double *p,int *mother,
				 int *Y, int **genotype, int n)
{
	int locus,j;
		
	for (j=1;j<=n;j++) {
		p[j]=0.0;
		for (locus=1;locus<=loci;locus++) {
			p[j] += glookup(genotype[j]+2*locus-2,Y+2*locus-2,mother+2*locus-2,locus);
		}
        p[j]=exp(-p[j]); 
	}
}
/*******************************************************************/
void add_geno(int *mother, int *geno, int *Y)
{
	int locus,add;
	for (locus=1;locus<=loci;locus++) {
		if (geno[2*locus]>0) continue;//can't do anything!!
		add=0;
		if (Y[2*locus-1]==Y[2*locus]) {
			if (mother[2*locus-1]==mother[2*locus]) add=Y[2*locus];
			else if (Y[2*locus]==mother[2*locus]||
				Y[2*locus]==mother[2*locus-1])  add=Y[2*locus];
		} else if (Y[2*locus-1]!=mother[2*locus-1]&&
			Y[2*locus-1]!=mother[2*locus]) {
			add= Y[2*locus-1];
		} else if (Y[2*locus]!=mother[2*locus-1]&&
			Y[2*locus]!=mother[2*locus]) {
			add= Y[2*locus];
		}
		if (add>0) {
			if (geno[2*locus-1]==0) {
				geno[2*locus-1]=add;
			} else if (geno[2*locus]==0) {
				if (geno[2*locus-1]!=add)
					geno[2*locus]=add;
			}
		}
	}
}
/*******************************************************************/
double randparentagefromdifferences(int *mother, int *parentage,int n)
{
	double lp=0.0,*p,pr;
	int i,**genotype;

	if (n==1) {
		for (i=1;i<=samples;i++) parentage[i]=1;
		return 0.0;
	}
	genotype=nrimatrix0(1,n,1,2*loci);
	p=nrdvector(1,n);
	for (i=1;i<=samples;i++) {
		getparprobs(p,mother,Y[i],genotype,n);
		parentage[i]=gen_from_probs2(p,n,&pr);
		lp+=log(pr);
		add_geno(mother,genotype[parentage[i]],Y[i]);
	}	
	free_dvector(p,1);
	free_imatrix(genotype,1,1);
	return lp;
}
/*******************************************************************/
double lprandparentagefromdifferences(int *mother, int *parentage,int n)
{
	double lp=0.0,*p;
	int i,**genotype;

	if (n==1) return 0.0;
	genotype=nrimatrix0(1,n,1,2*loci);
	p=nrdvector(1,n);
	for (i=1;i<=samples;i++) {
		getparprobs(p,mother,Y[i],genotype,n);
		lp += log(get_prob(parentage[i],p,n));
		add_geno(mother,genotype[parentage[i]],Y[i]);
	}
	free_dvector(p,1);
	free_imatrix(genotype,1,1);
	return lp;
}
/**************************************************************/
double sample_chain(chain *c)//,double *pl)
// assumes that we already have mu and proploc
{
	double lp;
	int i;
	//sample males parentage

	c->mothers.n=1;
    for (i=1;i<=samples;i++) c->mothers.parentage[i]=1;
	if (c->mothers.fixed==0) lp = samplemother(c->mothers.genotype);
	else lp=0.0;
	lp += sample_n(&c->fathers);
	lp += randparentagefromdifferences(c->mothers.genotype[1],c->fathers.parentage,
		c->fathers.n);

	// sample genotypes

	lp += gibbsparentalgenotypes(&c->fathers,&c->mothers,
		c->temper,c->temper3,c->muindex);//,pl);
	lp += lfactrl(c->fathers.n);

	return lp;
}
/**************************************************************/
double lpsample_chain(chain *c)//, double *pl)
// assumes that we already have mu and proploc
{
	double lp;

	if (c->mothers.n!=1) return -1E100;
	if (c->mothers.fixed==0) lp = lpsamplemother(c->mothers.genotype);
	else lp=0.0;
	lp += nparentsprior(&c->fathers);
	lp += lprandparentagefromdifferences(c->mothers.genotype[1],c->fathers.parentage,
		c->fathers.n);
	lp += lpselectedparentalgenotypes(&c->fathers,&c->mothers,
		c->temper,c->temper3,c->muindex);//,pl);
	lp += lfactrl(c->fathers.n);
	return lp;
}
/**************************************************************/
void gens2(int *a,int num, int l)
{
	int i;
	for (i=1;i<=l;i++) {
		if (num<=l-i+1) {
			a[1]=i;
			a[2]=i+num-1;
			return;
		}
		num-=l-i+1;
	}
}
int getnum2(int *x, int l)
{
	return x[2]+(x[1]-1)*(2*l-x[1])/2;
}
/**************************************************************/
void getrelprobs(double *p, int locus, int *use, int *mother)
{
	int al[3],i,pos;

	for (i=1;i<=alleles[locus]*(alleles[locus]+1)/2;i++) p[i]=0.0001;
	/* what is the probability if nothing is left   */
	
	for (i=1;i<=samples;i++) {
		for (pos=1,al[1]=1;al[1]<=alleles[locus];al[1]++) {
			for (al[2]=al[1];al[2]<=alleles[locus];al[2]++,pos++) {
				if (use[i]) p[pos] += (double)(!lookup_muts(Y[i]+2*locus-2,al,mother));
				else p[pos] += 0.01*(double)(!lookup_muts(Y[i]+2*locus-2,al,mother));
			}
		}
}
//	for (pos=1,al[1]=1;al[1]<=alleles[locus];al[1]++) {
//		for (al[2]=al[1];al[2]<=alleles[locus];al[2]++,pos++) {
//			printf("locus-%d:  mother %d %d    %d %d count %g\n",locus,
//				allele_labels[locus][mother[1]],
//				allele_labels[locus][mother[2]],
//				allele_labels[locus][al[1]],
//				allele_labels[locus][al[2]],p[pos]);
//		}
//	}
			

//	write_dvector(stdout," ",p,1,alleles[locus]*(alleles[locus]+1)/2);
//	for (i=1;i<=alleles[locus]*(alleles[locus]+1)/2;i++)
//		p[i]=exp(10*p[i]);
}
/**************************************************************/
/*  Need to fix this to allow for the possiblity of
    having fixed fathers - and allowing for resampling
	in that case                          */
/**************************************************************/
/*   Fixed safe !!!                                           */
/**************************************************************/
double sample_fathers(int **father, int n, int *mother, int fixed)
{
	int i,locus,*use,*finished,j,nf;
	double lp=0.0,*p,psel;
	
	p=nrdvector(1,max_alleles*(max_alleles+1)/2);
	use=nrivector0(1,samples);
	finished=nrivector0(1,samples);

	for (i=1;i<=n;i++) {
		for (j=1;j<=samples;j++) {
			if (finished[j]) use[j]=0;
			else use[j]=1;
		}
		for (locus=1;locus<=loci;locus++) {
			if (i>fixed) {
				getrelprobs(p,locus,use,mother+2*locus-2);
				nf=gen_from_probs2(p,alleles[locus]*(alleles[locus]+1)/2,&psel);
				lp += log(psel);
				gens2(father[i]+2*locus-2,nf,alleles[locus]);
			}
			for (j=1;j<=samples;j++) {
				if (use[j]) 
					if (lookup_muts(Y[j]+2*locus-2,mother+2*locus-2,father[i]+2*locus-2)>0)
						use[j]=0;
			}
		}
		for (j=1;j<=samples;j++) if (use[j]) finished[j]=1;
	}
	free_ivector(finished,1);
	free_ivector(use,1);
	free_dvector(p,1);
	return lp;
}
/**************************************************************/
double lpsample_fathers(int **father, int n, int *mother,int fixed)
{
	int i,locus,*use,j,nf;
	double lp=0.0,*p;
	
	p=nrdvector(1,max_alleles*(max_alleles+1)/2);
	use=nrivector0(1,samples);
	for (i=1;i<=n;i++) {
		for (j=1;j<=samples;j++) if (use[j]==0) use[j]=1;
		for (locus=1;locus<=loci;locus++) {
			if (i>fixed) {
				getrelprobs(p,locus,use,mother);
				nf = getnum2(father[i]+2*locus-2,alleles[locus]);
				lp += log(get_prob(nf,p,alleles[locus]*(alleles[locus]+1)/2));
			}
			for (j=1;j<=samples;j++) {
				if (use[j]) 
					if (lookup_muts(Y[j]+2*locus-2,mother+2*locus-2,father[i]+2*locus-2)>0)
						use[j]=0;
			}
		}
	}
	free_ivector(use,1);
	free_dvector(p,1);
	return lp;
}
/**************************************************************/
double sample_chainb(chain *c)//,double *pl)
// assumes that we already have mu and proploc
{
	double lp;
	int i;
//	char message[100];
	//sample males parentage

	c->mothers.n=1;
    for (i=1;i<=samples;i++) c->mothers.parentage[i]=1;
	c->mothers.offspring[1]=samples;
	if (c->mothers.fixed==0) lp = samplemother(c->mothers.genotype);
	else lp=0.0;
	lp += sample_n(&c->fathers);
	lp += sample_fathers(c->fathers.genotype,c->fathers.n,
		c->mothers.genotype[1],c->fathers.fixed);

	// sample genotypes
// Note I need another function instead of this - 
	// a good one may be to sample the three most
	// likely any then fill in the rest  - or something
    lp += c->sampleallparentaloffspring(&c->fathers,&c->mothers,1.0,1);//,pl);
	lp += lfactrl(c->fathers.n);
	
//	for (i=1;i<=c->fathers.n;i++) {
//		sprintf(message,"father %d  ",i) ;
//		print_geno(stdout,message,c->fathers.genotype[i]);
//	}
	
	return lp;
}
/**************************************************************/
double lpsample_chainb(chain *c)//, double *pl)
// assumes that we already have mu and proploc
{
	double lp;

	if (c->mothers.n!=1) return -1E100;
	if (c->mothers.fixed==0) lp = lpsamplemother(c->mothers.genotype);
	else lp=0.0;
	lp += nparentsprior(&c->fathers);
	//sample genotypes
	lp += lpsample_fathers(c->fathers.genotype,c->fathers.n,
c->mothers.genotype[1],c->fathers.fixed);
	//sample offspring
	lp += lpsampleallparentaloffspring(&c->fathers,&c->mothers,1.0,1);//,pl);
	lp += lfactrl(c->fathers.n);
	return lp;
}
/**************************************************************/
/**************************************************************/
/**************************************************************/
/**************************************************************/
/**************************************************************/
/**************************************************************/
int **getpatcount(int *m)
{
	int **tmp,i,add=0;
	tmp=nrimatrix0(1,alleles[1],1,alleles[1]);
	for (i=1;i<=samples;i++) {
		if (Y[i][1]==Y[i][2]) {
			add=Y[i][1];
		} else {
			if (Y[i][1]!=m[1]&&Y[i][1]!=m[2]) add= Y[i][1];
			else if (Y[i][2]!=m[1]&&Y[i][2]!=m[2]) add= Y[i][2];
		}
		if (add) {
			tmp[add][add]+=1;
		} else {
			if (Y[i][1]>Y[i][2]) tmp[Y[i][2]][Y[i][1]]+=1;
			else  tmp[Y[i][1]][Y[i][2]]+=1;
		}
	}
	return tmp;
}
/**************************************************************/
double sample_fathersc(int **genotype, int n, int *mg)
{
	int **patcount,i,j;
	double *p,pr;
	double lp=0.0;

	patcount=getpatcount(mg);
	p=nrdvector(1,alleles[1]);
	for (i=1;i<=alleles[1];i++) {
		p[i]=(double)patcount[i][i]+0.1;
		for (j=i+1;j<=alleles[1];j++) 
			p[i]+=0.5*(
			(double)patcount[i][j]
			);
	}
	for (i=1;i<=n;i++) {
		genotype[i][1]=gen_from_probs2(p,alleles[1],&pr);
		lp+=log(pr);
	//	p2=getbprobs(p,genotype[i][1]);
		genotype[i][2]=gen_from_probs2(p,alleles[1],&pr);
		lp+=log(pr);
		if (p[genotype[i][1]]>p[genotype[i][2]]) {
			p[genotype[i][1]] -= p[genotype[i][2]]-0.1;
			p[genotype[i][2]] -= p[genotype[i][2]]-0.1;
		} else {
			p[genotype[i][1]] -= p[genotype[i][1]]-0.1;
			p[genotype[i][2]] -= p[genotype[i][1]]-0.1;
		}
	}
	free_dvector(p,1);
	free_imatrix(patcount,1,1);
	return lp;
}
/**************************************************************/
double lpsample_fathersc(int **genotype, int n, int *mg)
{
	int **patcount,i,j;
	double *p;
	double lp=0.0;

	patcount=getpatcount(mg);
	p=nrdvector(1,alleles[1]);
	for (i=1;i<=alleles[1];i++) {
		p[i]=(double)patcount[i][i]+0.1;
		for (j=i+1;j<=alleles[i];j++) 
			p[i]+=0.5*(double)patcount[i][j];
	}
	for (i=1;i<=n;i++) {
		lp+=log(get_prob(genotype[i][1],p,alleles[1]));
		lp+=log(get_prob(genotype[i][2],p,alleles[1]));
		if (p[genotype[i][1]]>p[genotype[i][2]]) {
			p[genotype[i][1]] -= p[genotype[i][2]]+0.1;
			p[genotype[i][2]] -= p[genotype[i][2]]+0.1;
		} else {
			p[genotype[i][1]] -= p[genotype[i][1]]+0.1;
			p[genotype[i][2]] -= p[genotype[i][1]]+0.1;
		}
	}
	free_dvector(p,1);
	free_imatrix(patcount,1,1);
	return lp;
}

/**************************************************************/
double sample_chainc(chain *c)//,double *pl)
// assumes that we already have mu and proploc
{
	double lp;
	int i;

	if (loci>1) myerror("sample c is only for a single locus");

	c->mothers.n=1;
    for (i=1;i<=samples;i++) c->mothers.parentage[i]=1;
	if (c->mothers.fixed==0) lp = samplemother(c->mothers.genotype);
	else lp=0.0;
	lp += sample_n(&c->fathers);
	lp += sample_fathersc(c->fathers.genotype,c->fathers.n,c->mothers.genotype[1]);
    lp += sampleallparentaloffspringa(&c->fathers,&c->mothers,1.0,1);//,pl);
	lp += lfactrl(c->fathers.n);

	return lp;
}
/**************************************************************/
double lpsample_chainc(chain *c)//, double *pl)
// assumes that we already have mu and proploc
{
	double lp;
	if (loci>1) myerror("sample c is only for a single locus");
	if (c->mothers.n!=1) return -1E100;
	if (c->mothers.fixed==0) lp = lpsamplemother(c->mothers.genotype);
	else lp=0.0;
	lp += nparentsprior(&c->fathers);
	lp += lpsample_fathersc(c->fathers.genotype,c->fathers.n,c->mothers.genotype[1]);
	lp += lpsampleallparentaloffspring(&c->fathers,&c->mothers,1.0,1);//,pl);
	lp += lfactrl(c->fathers.n);
	return lp;
}
/**************************************************************/
